/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Callback process
*  SCOPE :              Callback process
*  FILE :               callback.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

/* IMPORTS *******************************************************************/

#ifndef __iostream_H
#define __iostream_H
#include <iostream.h>
#endif

#ifndef __signal_H
#define __signal_H
#include <signal.h>
#endif

#ifndef _oid_HXX
#include "oid.hxx"
#endif

#ifndef _pagehdl_HXX
#include "pagehdl.hxx"
#endif

#ifndef _pollset_HXX
#include "pollset.hxx"
#endif

#ifdef __GNUG__
#pragma implementation "callbackclient_svc.hxx"
#endif
#ifdef __GNUG__
#pragma implementation "callbackserver_svc.hxx"
#endif

#include "callbackclient_svc.hxx"
#include "callbackserver_svc.hxx"

/*****************************************************************************/

CallbackServer svcServer ;
CallbackClient svcClient(AF_UNIX) ;
PageSet* KeepList ;


void CB_Terminate() {
  svcServer.close() ;
  svcClient.close() ;
  exit(1) ;
}

void CB_linkBreak(SvcSocket*) {
  svcServer.close() ;
  svcClient.close() ;
  exit(1) ;
}


void CallbackClient::kill() 
{
  CB_Terminate() ;
}


void CallbackClient::reset() 
{
  KeepList->clear() ;
}


void CallbackClient::keepLock(const ListVal<PID>& l) 
{
  KeepList->clear() ;
  IterListVal<PID> iter(*(ListVal<PID>*)&l) ;
  for (; iter.more(); iter.next()) {
    (*KeepList) += new PageHdl(iter.get()) ;
  }
}


int CallbackClient::askLock(const PID& pid) 
{
  if ((*KeepList)[pid] != NULL) {
    (*KeepList) -= pid;
    return TRUE ;
  } else {
    return FALSE ;
  }
}


int CallbackServer::askLock(const PID& pid) 
{
  if ((*KeepList)[pid] != NULL) {
    (*KeepList) -= pid;
    return TRUE ;
  } else {
    return FALSE ;
  }
}


int main(int argc, char * argv[])
{
  if( argc < 3) {
    cerr << "error: usage : " << argv[0] << " <filter> <pid> \n";
    exit(4);
  }

  setsid() ;
  
  signal(SIGKILL, ((void (*) (int)) CB_Terminate));
  signal(SIGINT,  ((void (*) (int)) CB_Terminate));
  signal(SIGQUIT, ((void (*) (int)) CB_Terminate));
  signal(SIGTERM, ((void (*) (int)) CB_Terminate));

  SvcInit() ;

  PollSet    ss;
  KeepList = new PageSet ;    
  
  svcClient.export(argv[1]) ;
  svcServer.export(argv[1]) ;
  svcClient.linkBreak(CB_linkBreak) ;
  
  int pid = atoi(argv[2]) ;
  
  ss += &svcServer ;
  ss += &svcClient ;
  while (1) {
    ss.exec(30000) ;
    if (kill(pid, SIGUSR1) == -1) {
      break ;
    }
  }
}


