%{

/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
  
#ifndef __rename_H
#include "rename.h"
#endif
  
#ifndef __string_H
#define __string_H
#include <string.h>
#endif

#ifndef __stdlib_H
#define __stdlib_H
#include <stdlib.h>
#endif

#ifndef __iostream_H
#define __iostream_H
#include <iostream.h>
#endif

#ifndef __unistd_H
#define __unistd_H
#include <unistd.h>
#endif

#ifdef __alpha
#include "alphadecl.h"
#endif

#ifndef __list_HXX
#include "list.hxx"
#endif

#ifndef __var_H
#include "var.h"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#include "delvol.h"

static int Recover = FALSE ;
int delv_linecount = 0 ;


extern "C" {
  int yywrap() ;
  int yyparse() ;
}

#if defined(__alpha__) || (defined(__sun__) && !defined(__svr4__))
int yylex() ;
#endif

#if defined(__alpha__) || (defined(__sun__) && !defined(__svr4__))
int yyoutput(int c) ; 
#else
extern "C" { int yyoutput(int c) ; }
#endif


extern void yoo_repert(const char*) ;

extern FILE* yyin ;
FILE* f_delvol ;


%}

%union  {
  int  integer ;
  char text[SIZEOF_TOKEN];
} ;

%token VOLUME FILETOK LOG SERVER ID
%token <text> IDENT FILENAME
%token <integer> INT

%start all

%%

all: server_list ;


server_list :
         server
         | server server_list ;


server :
         SERVER IDENT '{' log volume_list '}' ;

log :
         LOG FILENAME ';' 
           {
	     fprintf(f_delvol, "rm -f %s/.yoo_%s_*_log\n", $2, $<text>-1) ;
           }

volume_list :
         volume
 	 | volume volume_list ;

volume :
         VOLUME IDENT '{' ID INT ';' FILETOK FILENAME ';' '}'
           {
	     char lockfile[512] ;
	     sprintf(lockfile, "%s.LCK", $8) ;
  	     int fd = open(lockfile, O_RDWR) ;
             if (fd != -1) {
	       SYSCALLIMESS(lockf(fd, F_TLOCK, 0), "volume already used") ;
	     }
             close(fd) ;
                
	     if ( (Recover == FALSE) || (strcmp("admin", $2) != 0) ) {
	       fprintf(f_delvol, "rm -f %s\n", $8) ;
	       fprintf(f_delvol, "rm -f %s.LCK\n", $8) ;
	     }
	   }

%%


void delvol(char* name, int recover) {


  Recover = recover ;
  yoo_env(name, 0) ;
  yoo_repert() ;

  char tmp[256] ;
  f_delvol = fopen(tmpnam(tmp), "w") ;

  if (f_delvol == NULL) {
    perror("Error : Cannot create temporary file") ;
    unlink(tmp) ;
    exit(1) ;
  }

  yyin = fopen(YOO_VOL_FILE, "r") ;
  if (yyin != NULL) {
    char res[500] ;
    do {
      if (recover) {
	cout << "Delete base \"" << YOODA_BASE << "\" (keep schema) [y/n] ? ";
      } else {
	cout << "Delete base \"" << YOODA_BASE << "\" [y/n] ? ";
      }
      cin >> res ;
    } while ((strcmp(res,"y")!=0) && (strcmp(res,"n")!=0)) ;
    
    if (strcmp(res,"n") == 0) {
      unlink(tmp) ;
      exit(1) ;
    }
    yyparse() ;
    fclose(yyin) ;
    fprintf(f_delvol, "rm -f %s\n", YOO_VOL_FILE) ;
    fclose(f_delvol) ;
    char command[256] ;
    sprintf(command, "chmod u+x %s; %s", tmp, tmp) ;
    system(command) ;
  } else {
    fclose(f_delvol) ;
  }
  unlink(tmp) ;
}

void yyerror(char* s) {
  cout << "Volume parse error line : " << delv_linecount << endl ;
  yoo_exit(1) ;
}
