/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <stdio.h>
#include "monitor.hxx"
#include "monitor_clnt.hxx"



void option_error(char* name) 
{
  printf("usage: %s [-v] [-b <basename>] <command>\n", name);
  printf("command = lb = list bases |\n") ;
  printf("          lt = list clients (need base) |\n") ;
  printf("          ls = list servers (need base) |\n") ;
  printf("          kt <pid> = kill client (need base) |\n") ;
  printf("          ks <server-name> = kill server (need base) \n") ;
  printf("          kas = kill all servers (need base) \n") ;	
  exit(1) ;
}

int main(int argc, char** argv) {

  typedef enum { NONE, LIST_BASES, LIST_CLIENTS, LIST_SERVERS,
		 KILL_CLIENT, KILL_SERVER, KILL_ALL_SERVERS } Action ;

  Action action = NONE ;
  char* object = NULL ;
  char* base = NULL ;
  int vid = 2 ;
  int vers = 0 ;
  int repair = 0 ;
  
  for (int i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-b") == 0) {
      if (i+1 < argc) {
	base = argv[i+1] ;
	i++ ;
      } else { option_error(argv[0]) ; }
    } else if (strcmp(argv[i], "-v") == 0) {
      vers = 1 ;
    } else if ((action==NONE) && (strcmp(argv[i], "lb") == 0)) {
      action = LIST_BASES ;
    } else if ((action==NONE) && (strcmp(argv[i], "lt") == 0)) {
      action = LIST_CLIENTS ;
    } else if ((action==NONE) && (strcmp(argv[i], "ls") == 0)) {
      action = LIST_SERVERS ;
    } else if ((action==NONE) && (strcmp(argv[i], "kt") == 0)) {
      action = KILL_CLIENT ;
      if (i+1 < argc) {
	object = argv[i+1] ;
	i++ ;
      } else { option_error(argv[0]) ; }
    } else if ((action==NONE) && (strcmp(argv[i], "ks") == 0)) {
      action = KILL_SERVER ;
      if (i+1 < argc) {
	object = argv[i+1] ;
	i++ ;
      } else { option_error(argv[0]) ; }
    } else if ((action==NONE) && (strcmp(argv[i], "kas") == 0)) {
      action = KILL_ALL_SERVERS ;
    } else {
      option_error(argv[0]) ;
    }
  }

  if ((base == NULL) && (action != LIST_BASES)) {
    option_error(argv[0]) ;
  }

  SvcInit() ;
  
  switch (action) {
  case NONE :
    option_error(argv[0]) ;
    break ;
  case LIST_BASES :
    display_bases(list_bases()) ;
    break ;
  case LIST_SERVERS :
    display_servers(base, list_servers(base)) ;
    break ;
  case KILL_SERVER :
    kill_server(base, object) ;
    cout << "\n\tserver " << object << " killed" << endl ;
    break ;
  case KILL_ALL_SERVERS :
    kill_all_servers(base) ;
    cout << "\n\tAll servers killed for database <" << base << ">" << endl;
    break ;
  case LIST_CLIENTS :
    display_clients(list_clients(base)) ;
    break ;
  case KILL_CLIENT :
    kill_client(base, atoi(object)) ;
    break ;
  }
}
    
