#include <stdlib.h>
#include <stream.h>
#include <stdio.h>

#include <stdio.h>
#include <string.h>
#include <stream.h>
#include <sys/mman.h>

#include "clock.h"
#include "obj.hxx"
#include "cluster.hxx"
#include "smartp.hxx"
#include "yoo_root.hxx"

#include "ref_Part.hxx"
#include "part.hxx"


/*************************************************/
/***************************************************/

void create_part(int partNb, const Ref< ListX<Part> >& parts) {

  timeBegin();
  
  int baseSize = 0 ;

  for(int i = 0; i < partNb; i++) {
    Ref<Part> p = new Part ;
    p->id = i  + baseSize ;
    (*parts) += p ;
  }

  cout << "Nodes created" << endl ;

  timeEnd();
  cout << "Node Creation time = " << timeDiff() << " sec" << endl ;
  timeBegin() ;
  
  baseSize += partNb ;

  for (i = baseSize-partNb; i < baseSize; i++) {
    for (int j = 0; j < 3; j++) {
      int cpart ;
      if (rand()%10 > 0) {
	// 90% of time create connection to the closest 1% of parts
	cpart = i + (rand()%(baseSize/100)) - (baseSize/200);
	// double up at the ends so stay in part_id range
        if (cpart < (baseSize/200))
	  cpart = cpart + (baseSize/200);
	else if (cpart > (baseSize - (baseSize/200)))
	  cpart = cpart - (baseSize/200);
      } else
	// 10 % of time create connection to any part 0 .. (baseSize - 1)
	cpart = (rand()%baseSize);
	
      // connect the part n to cpart

      Part* from = (*parts)[i] ;
      Part* to = (*parts)[cpart] ;
      from->connect[j] = to ;
      to->revConnect += from ;
    }
  }

  cout << "Nodes connected" << endl ;

  timeEnd();
  cout << "Nodes connection commit time = " << timeDiff() << " sec" << endl ;

  timeBegin();
  yoo_commit() ;
  timeEnd();
  cout << "Creation Commit time = " << timeDiff() << " sec" << endl ;
}

void lookup(int partNb, const Ref< ListX<Part> >& parts) {

  timeBegin();
  for(int i = 0; i < partNb; i++) {
    (*parts)[(rand()%20000)]->foo() ;
  }
  timeEnd();
  cout << "lookup number " << " : " << timeDiff() << " sec\n";
} 

void traversal(Part* orig, int depth, int upd) {

  if ( (upd != -1) && (rand()%10 < upd) ) {
    orig->x++ ; orig->y++ ;
  }

  if (depth--) {
    traversal(orig->connect[0], depth, upd) ;
    traversal(orig->connect[1], depth, upd) ;
    traversal(orig->connect[2], depth, upd) ;
  }
}

int reverse_traversal(Part* orig, int depth) {

  int visited = orig->revConnect.count() ;
  Iterator<Part> iter(orig->revConnect) ;
  for (; iter.more(); iter.next()) {
    if (depth > 1)
      visited += reverse_traversal(iter.get(), depth-1) ;
  }
  return(visited) ;
}


int main(int argc, char** argv) {

  Ref< ListX<Part> > parts ;
  double totalTime = 0 ;
  char* serv = getenv("SERVER") ;

  yoo_connect("oo1", serv) ;

  if((argc == 3) && (argv[1][0] == 'c')) {
    int size = atoi(argv[2]) ; // number of objects to load in the database
    parts = new ListX<Part> ;
    create_part(size, parts) ;
    yoo_setname(parts, "bench") ;
  } 
  else if(argc >= 4) {
    yoo_getname(parts, "bench") ;
    int size = atoi(argv[2]) ;//number of objects to insert, lookup or traverse
    int loop = atoi(argv[3]) ;// number of time to call bench
    if (argv[1][0] == 'i') {
      for(int j = 0; j < loop; j++) {
        timeBegin();
        create_part(size, parts) ;
        timeEnd();
        cout << "insert number " << j  << " : " << timeDiff() << " sec\n\n";
      }
      yoo_commit() ;      
    } else if (argv[1][0] == 'l') {
      for(int j = 0; j < loop; j++) {
        lookup(size, parts) ;
      }
    } else if (argv[1][0] == 't') {
      int baseSize = parts->count() ;
      int n = (rand()%baseSize) ;    
      int hot = atoi(argv[4]) ;
      int upd = atoi(argv[5]) ;
      int commit = atoi(argv[6]) ;
      int travers = 0 ;
      Part* p = (*parts)[n] ;
      timeBegin();
      traversal(p, size, upd) ;
      timeEnd();
      totalTime += timeDiff() ;
      if (commit) {
	yoo_commit() ;
      }
      cout << "traversal Cold number " << 1 << "\n" << travers << " parts\n"
	   << "Time = " << timeDiff() << " sec\n\n" << endl ; 
      for(int j = 0; j < loop; j++) {
	travers = 0 ;
	int n = (rand()%baseSize) ;
	Part* p = (*parts)[n] ;
	timeBegin();
	traversal(p, size, upd) ;
	if (commit) {
	  yoo_commit() ;
	}
	timeEnd();
	totalTime += timeDiff() ;
	cout << "traversal Warm number " << j << "\n" << travers << " parts\n"
	     << "Time = " << timeDiff() << " sec\n\n" << endl ; 
      }
      travers = 0 ;
      for(j = 0; j < hot; j++) {
	Part* p = (*parts)[n] ;
	timeBegin();
	traversal(p, size, upd) ;
	if (commit) {
	  yoo_commit() ;
	}
	timeEnd();
	totalTime += timeDiff() ;
	cout << "traversal hot number " << j << "\n" << travers << " parts\n"
	     << "Time = " << timeDiff() << " sec\n\n"; 
      }
    } else
      cerr << "bad command \n" ;
   } else
      cerr << "bad command \n" ;

  timeBegin();
  yoo_commit() ;
  timeEnd();
  totalTime += timeDiff() ;
  cout << "Total time = " << totalTime << " sec" << endl ;

  yoo_disconnect() ;
}


