/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <string.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "VarParams.h"


extern int	debugMode;
extern char*	types[NumTypes];


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Constructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::AtomicPart(int ptId) : id(ptId)
{
    if (debugMode) {
        printf("BEGIN AtomicPart::AtomicPart(ptId = %d)\n", ptId);
    }
    
    // initialize internal state of new part (most of it randomly)
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAtomicDate +
    		(int) (random() % (MaxAtomicDate-MinAtomicDate+1));
    x = (int) (random() % XYRange);
    y = (int) (random() % XYRange);

    docId = (int) random() % TotalCompParts + 1;

    if (debugMode) {
        printf("MIDDLE AtomicPart::AtomicPart(ptId = %d)\n", ptId);
        printf("\tbuildDate = %d\n", buildDate);
        printf("\tx, y = %d, %d\n", x, y);
    }

    // add part to AtomicPart extent
    AtomicPart::extent_id->insert(this) ;
    AtomicPart::extent_date->insert(this) ;

    this->to.init(NumConnPerAtomic) ;
    
    if (debugMode) {
        printf("END AtomicPart::AtomicPart(ptId = %d)\n", ptId);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart swapXY Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::swapXY()
{
    if (debugMode) {
        printf("                    AtomicPart::swapXY(ptId = %d, x = %d, y = %d)\n", id,x,y);
    }

    // exchange X and Y values

    int tmp = x;
    x = y;
    y = tmp;

    if (debugMode) {
        printf("                    [did swap, so x = %d, y = %d]\n", x, y);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart toggleDate Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::toggleDate()
{
    if (debugMode) {
        printf("                    AtomicPart::toggleDate(ptId = %d, buildDate = %d)\n",
	       id, buildDate);
    }

    // increment build date if odd, decrement it if even

    AtomicPart::extent_date->remove(this) ;      
    if (buildDate % 2) {
	// odd case
      buildDate++;
    } else {
	// even case
	buildDate--;
    }
    AtomicPart::extent_date->insert(this) ;      

    if (debugMode) {
        printf("                    [did toggle, so buildDate = %d]\n", buildDate);
    }
}

////////////////////////////////////////////////////////////////////////////
//
// AtomicPart DoNothing Method 
//
////////////////////////////////////////////////////////////////////////////

int RealWork; // set to one to make DoNothing do work.
int WorkAmount; // amount of work to do

void AtomicPart::DoNothing()
{
    struct timeval tv;
    struct timezone tz;

    int i;
    if (id < 0)
	   printf("DoNothing - negative id\n");

    if (debugMode) {
        printf("==> DoNothing(x = %d, y = %d, type = %s)\n", x, y, type);
    }

    if (RealWork) {
            for (i = 0; i < WorkAmount; i++) {
               gettimeofday(&tv, &tz);
            }
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Destructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::~AtomicPart()
{
    if (debugMode) {
        printf("Entering AtomicPart::~AtomicPart(id = %d)\n", id);
    }

    // delete all affected connections
if (debugMode) {
    printf("    In ~AtomicPart, about to delete 'to' connections\n");
}
    Iterator<Connection> iter(this->to) ;
    for (; iter.more(); iter.next()) {
      AtomicPart* atom = iter.get()->to ;
      int pos = atom->from.member(iter.get()) ;
      atom->from.remove(pos) ;
      delete (Connection*)iter.get();
    }
if (debugMode) {
    printf("    In ~AtomicPart, about to delete 'from' connections\n");
}
    Iterator<Connection> iter2(this->from) ;
    for (; iter2.more(); iter2.next()) {
      AtomicPart* atom = iter2.get()->from ;
      int pos = atom->to.member(iter2.get()) ;
      atom->to.remove(pos) ;
      delete (Connection*)iter2.get();
    }

    // remove part from AtomicPart extent
if (debugMode) {
    printf("    In ~AtomicPart, about to remove part from AtomicPart::extent\n");
}
    AtomicPart::extent_id->remove(this) ;
    AtomicPart::extent_date->remove(this) ;

    if (debugMode) {
        printf("Leaving AtomicPart::~AtomicPart(id = %d)\n", id);
    }
}

