/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Insert - Create NumNewCompParts new composite parts (which
// means also creating a set of AtomicParts for each.)  Add
// a reference to each new composite part from a randomly chosen
// base assembly (a different random choice of base assembly for
// each new composite part.)
//
//////////////////////////////////////////////////////////////////


extern int	debugMode;
extern int	nextAtomicId;
extern int	nextCompositeId;
extern int	nextBaseAssemblyId;


//////////////////////////////////////////////////////////////////
//
// Routine to do Insert
//
//////////////////////////////////////////////////////////////////


void insert1()
{

    // create the desired number of new composite parts, adding each
    // one as a (private) composite part that a randomly selected base
    // assembly now wishes to use

    int compId, assmId;
    CompositePart* comp;

    if (debugMode) {
	printf("In Insert, nextCompositeId = %d:\n", nextCompositeId);
	printf("In Insert, nextAtomicId = %d:\n", nextAtomicId);
	printf("In Insert, nextBaseAssemblyId = %d:\n", nextBaseAssemblyId);
    }

    for (int i = 0; i < NumNewCompParts; i++) {

	// add a new composite part to the design library

        if (debugMode) {
	    printf("In Insert, making composite part %d:\n", nextCompositeId);
	}
        compId = nextCompositeId++;
        comp = new CompositePart (compId);

	// randomly select a base assembly that should use it

	assmId = (int) (random() % (nextBaseAssemblyId-1)) + 1;

	// locate the appropriate base assembly object

        BaseAssembly* assm = BaseAssembly::extent->getFirst(assmId) ;

        // finally, add the newly created composite part as a privately used
        // member of the randomly selected base assembly

	assm->componentsPriv += comp;
	comp->usedInPriv += assm ;

        if (debugMode) {
	    printf("[just made it be used by base assembly %d]\n", assmId);
        }
    }
}

