/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdlib.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"
#include "baidlist.h"

extern int RealWork;         // set to one to make DoNothings do work.
extern int WorkAmount;		// controls how much work DoNothings do

extern void SetParams(char* configFileName);

extern double ComputeWallClockTime(struct timeval *startWallTime, 
			           struct timeval *endWallTime);

extern double ComputeUserTime(struct rusage *startUsage, 
	        	      struct rusage *endUsage);

extern double ComputeSystemTime(struct rusage *startUsage, 
	 	                struct rusage *endUsage);


//////////////////////////////////////////////////////////////////
//
// Global Variables, etc., for Benchmarking ODB Operations
//
//////////////////////////////////////////////////////////////////

int	nextAtomicId = 0;
int	nextCompositeId = 0;
int	nextComplexAssemblyId = 0;
int	nextBaseAssemblyId = 0;
int	nextModuleId = TotalModules;
int	debugMode = FALSE;

struct rusage startUsage, endUsage;
struct timeval startWallTime;
struct timeval endWallTime;
struct timezone ignoreTimeZone;
struct timeval startWarmTime;

Ref<DicoAtomicPartId> AtomicPart::extent_id ;
Ref<DicoAtomicPartDate> AtomicPart::extent_date ;
Ref<DicoDocument> Document::extent ;
Ref<DicoCompositePart> CompositePart::extent ;
Ref<DicoBaseAssembly> BaseAssembly::extent ;
Ref<DicoModule> Module::extent ;

#include "dictionary.hxx"

Ref<Cluster> DocCluster ;
char* DocCluName = "DocCluName" ;

char    *types[NumTypes] = {
		"type000", "type001", "type002", "type003", "type004",
		"type005", "type006", "type007", "type008", "type009"
	};

// here only to keep the make happy. used in gendb
// and in Insert!!!
BAIdList* private_cp;
BAIdList* shared_cp;


main(int argc, char** argv)
{

        char resultText[200];  // to hold result message in order to avoid
                               // printf in timing section.

	//////////////////////////////////////////////////////////////////
	//
	// Initialize
	//
	//////////////////////////////////////////////////////////////////

	// initialize parameters for benchmark.
	SetParams(argv[1]);


        // need to malloc space for these, but don't need to store
        // any info since Insert looks up assemblies "by hand".
	shared_cp = new BAIdList[TotalCompParts+NumNewCompParts+1];
	private_cp = new BAIdList[TotalCompParts+NumNewCompParts+1];


	// Compute structural info needed by the update operations,
        // since these operations need to know which id's should
        // be used next.

	int baseCnt = NumAssmPerAssm;
	int complexCnt = 1;
	for (int i = 1; i < NumAssmLevels-1; i++) {
            baseCnt = baseCnt * NumAssmPerAssm;
            complexCnt += complexCnt * NumAssmPerAssm;
	}
	nextBaseAssemblyId = TotalModules*baseCnt + 1;
	nextComplexAssemblyId = TotalModules*complexCnt + 1;
	nextAtomicId = TotalAtomicParts + 1;
	nextCompositeId = TotalCompParts + 1;

	int opIndex = 2;
	int repeatCount = 1;
	BenchmarkOp whichOp = Trav1;
        int manyXACTS = 0;

	// Now open the database, disable whole-segment transfers (in favor
	// of cluster-level ones), and start up a transaction

	char *dbname = getenv("DBNAME");
	char *dbserver = getenv("DBSERVER");

	yoo_connect(dbname, dbserver) ;

	yoo_getname(AtomicPart::extent_id, "AtomicPart::extent_id") ;
	yoo_getname(AtomicPart::extent_date, "AtomicPart::extent_date") ;
	yoo_getname(Document::extent, "Document::extent") ;
	yoo_getname(CompositePart::extent, "CompositePart::extent") ;
	yoo_getname(BaseAssembly::extent, "BaseAssembly::extent") ;
	yoo_getname(Module::extent, "Module::extent") ;
	DocCluster = YoodaDictionary->dictClu.getFirst(DocCluName)->clu ;   

	int partId = 1 ;
	Iterator<AtomicPart> iterPart(*AtomicPart::extent_id) ;
	for (; iterPart.more(); iterPart.next()) {
	  if (iterPart.get()->id != partId++) {
	    cout << "Erreur index  AtomicPart::extent_id" << endl ;
	  }
	}
	if ((partId-1) != TotalAtomicParts) {
	  cout << "Erreur collection dans  AtomicPart::extent_id" << endl ;
	}

	int compId = 1 ;
	Iterator<CompositePart> iterComp(*CompositePart::extent) ;
	for (; iterComp.more(); iterComp.next()) {
	  if (iterComp.get()->id != compId++) {
	    cout << "Erreur index  CompositePart::extent" << endl ;
	  }
	}
	if ((compId-1) != TotalCompParts) {
	  cout << "Erreur collection dans  CompositePart::extent" << endl ;
	}

	int baseId = 1 ;
	Iterator<BaseAssembly> iterBase(*BaseAssembly::extent) ;
	for (; iterBase.more(); iterBase.next()) {
	  if (iterBase.get()->id != baseId++) {
	    cout << "Erreur index  BaseAssembly::extent" << endl ;
	  }
	}
	if ((baseId-1) != TotalBaseAssm) {
	  cout << "Erreur collection dans  BaseAssembly::extent" << endl ;
	}

	

        //////////////////////////////////////////////////////////////////
	//
	// Shutdown
	//
	//////////////////////////////////////////////////////////////////

	// close the database
	yoo_disconnect() ;


	// Exit
	exit(0);
}

