/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
*****************************************************************************/
/*****************************************************************************
*
*  MODULE :             Interface to DictX
*  SCOPE :              Interface to DictX
*  FILE :               dictx.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
*****************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __dictx_HXX
#define __dictx_HXX

/* IMPORTS ******************************************************************/

#ifndef __colx_HXX
#include "colx.hxx"
#endif

#ifndef __listx_HXX
#include "listx.hxx"
#endif

/****************************************************************************/

#define yoo_index(attr) (long)&attr

class DictXGen : public ObjectX, public LocalColX {

  public :
  
  static void* operator new(size_t s, Cluster* clu=YOO_DEF_CLU) { 
    return ObjectX::operator new(s, clu, UNTYPED) ;
  }

  static void* operator new(size_t s, ObjectX* obj) {
    return ObjectX::operator new(s, obj, UNTYPED) ;
  }

  // Get the element at position pos
  Ref<ObjectX> get(int pos) const { 
    if (this->count() == 0) {
      return NULL ;
    } else {
      return *(Ref<ObjectX>*)this->col()->getElem(pos) ;
    }
  }
  
  protected :

  // Initialise (if needed) a new collection
  void init() ;

  // Compare the dictionaries. Return 1 if every element of c is member of this
  int compare(const LocalColX& c,void* (*)(const void*),
	      int (*)(const void*,const void*)) const ; 

  // Insert every elements of the collection
  void insert(const LocalColX& l,void* (*)(const void*),
	      int (*)(const void*,const void*)); 

  // Insert element in the dictionary
  void insert(const yoo_ObjectX&,void* (*)(const void*),
	      int (*)(const void*,const void*)) ;

  // Insert every elements of the collection if key is uniq
  void insertUniq(const LocalColX& l,void* (*)(const void*),
	      int (*)(const void*,const void*)) ;

  // Insert element in the dictionary if key is uniq
  int  insertUniq(const yoo_ObjectX&,void* (*)(const void*),
	      int (*)(const void*,const void*)) ;

  // Get first element with the given key
  yoo_ObjectX getFirst(const void*,void* (*)(const void*),
		       int (*)(const void*,const void*)) const ;

  // Return a temporary listx which contains every objects of the given key
  Ref< ListX<ObjectX> > getAll(const void*,void* (*)(const void*),
	      int (*)(const void*,const void*)) const ;

  // Remove object from dictionary
  int remove(const yoo_ObjectX&, void* (*)(const void*),
	      int (*)(const void*,const void*)) ;

  // Remove every object which have the given key
  int removeAll(const void*, void* (*)(const void*),
	      int (*)(const void*,const void*)) ;

  // Test membership of the object. Return position or -1
  int member(const yoo_ObjectX&, void* (*)(const void*),
	      int (*)(const void*,const void*)) const ;

  // Return the indice of the first element which have the given key
  int indice1(const void*, void* (*)(const void*),
	      int (*)(const void*,const void*)) const ;
} ;


#include "dictx_tmpl.hxx"

yoo_declare(DictXGen) ;

#define yoo_declare_DictX(ELEM, KEYT, KEY) \
     template class Iterator<ELEM> ; \
     template class LocalListX<ELEM> ; \
     template class ListX<ELEM> ; \
     template class DictX<ELEM, KEYT, yoo_index(ELEM::KEY)> ; \
     template class Ref<DictX<ELEM, KEYT, yoo_index(ELEM::KEY)> > ;

#endif




