/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             sharedcol
*  SCOPE :              Management of shared collection
*  FILE :               sharedcol.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

/*=
[DESCRIPTION :]
=*/

/* IMPORTS *******************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

#ifndef _mman_H
#define _mman_H
#include <sys/mman.h>
#endif

#ifndef __sharedcol_HXX
#include "sharedcol.hxx"
#endif
#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif
#ifndef __pagehdlmem_HXX
#include "pagehdlmem.hxx"
#endif

ListPtr<GenSharedMod> GenSharedCol::listmod ;

void GenSharedCol_commitAll() { GenSharedCol::commitAll() ; }
void GenSharedCol_abortAll() { GenSharedCol::abortAll() ; }


/*= EXTERNAL FUNCTION =======================================================*/
GenSharedMod::GenSharedMod(const GenSharedCol& col) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->_newElemList = col.newElemList() ;
  this->_delElemTab = col._delElemTab ;
  this->_savElemList = col.savElemList() ;
  this->_func = col.registerCommit() ;
}



/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedMod::commit() 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if ((this->_newElemList == NULL) && (_delElemTab == NULL)) return ;
  
  this->_delElemList = new(YOO_TMP_CLU) ListX<ObjectX> ;

  if (this->_delElemTab != NULL) {
    for (int pos = 0; pos != -1; pos++) {
      pos = this->_delElemTab->look(pos,1) ;
      if (pos != -1) {
	(*this->_delElemList) += this->_savElemList->get(pos) ;
      } else {
	break ;
      }
    }
  }
  this->_func(this) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
GenSharedMod::~GenSharedMod()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  delete (DictXGen*) this->_newElemList ;
  delete (ListX<ObjectX>*) this->_delElemList ;
  delete this->_delElemTab ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedCol::commitAll()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (listmod.count() != 0) {
    yoo_upgrade() ;
    
    IterListPtr<GenSharedMod> iter(listmod) ;
    iter.reset() ;
    for (; iter.more(); iter.next()) {
      iter.get()->commit() ;
      delete iter.get() ;
    }
  }

  listmod.clear() ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedCol::abortAll()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  IterListPtr<GenSharedMod> iter(listmod) ;
  for (; iter.more(); iter.next()) {
    delete iter.get() ;
  }

  listmod.clear() ;
}


/*= EXTERNAL FUNCTION =======================================================*/
GenSharedCol::~GenSharedCol()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->_delElemTab != NULL) {
    delete (DictXGen*)this->newElemList() ;
    delete this->_delElemTab ;
  }
  IterListPtr<GenSharedMod> iter(listmod) ;
  for (; iter.more(); iter.next()) {
    if (iter.get()->_savElemList == this->savElemList()) {
      iter.remove() ;
      break ;
    }
  }
  delete (DictXGen*)this->savElemList() ;
  return ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int GenSharedCol::operator==(const GenSharedCol& col) const
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;

  if (this->count() != col.count()) return 0 ;
  
  GenSharedIterator iter(col) ;
  for (; iter.more(); iter.next()) {
    if (this->member(iter.get()) == -1) return 0 ;
  }

  return 1 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int GenSharedCol::insert(const LocalListX<ObjectX>& l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;

  int count = 0 ;
  Iterator<ObjectX> iter(l) ;
  for (; iter.more(); iter.next()) {
    if (this->insert(iter.get())) {
      count++ ;
    }
  }

  return count ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int GenSharedCol::insert(const GenSharedCol& col)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;

  int count = 0 ;
  GenSharedIterator iter(col) ;
  for (; iter.more(); iter.next()) {
    if (this->insert(iter.get())) {
      count++ ;
    }
  }

  return count ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int GenSharedCol::count() const
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;

  int size = this->newElemList()->count() + this->savElemList()->count() ;
  for (int pos = 0; pos != -1; pos++) {
    pos = this->_delElemTab->look(pos,1) ;
    if (pos != -1) {
      size-- ;
    } else {
      break ;
    }
  }
  return size ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedCol::clear()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;

  this->newElemList()->clear() ;
  this->_delElemTab->zero() ;
}


/*= EXTERNAL FUNCTION =======================================================*/
GenSharedIterator::GenSharedIterator(const GenSharedCol& c) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  c.init() ;
  
  _iterSav = new Iterator<ObjectX>(*c.savElemList()) ;
  _iterNew = new Iterator<ObjectX>(*c.newElemList()) ;
  _iterCurr = _iterNew->more() ? _iterNew : _iterSav ;
  _iterDel = c._delElemTab ;
}


/*= EXTERNAL FUNCTION =======================================================*/
GenSharedIterator::~GenSharedIterator()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  delete _iterSav ;
  delete _iterNew ;
}


/*= EXTERNAL FUNCTION =======================================================*/
Ref<ObjectX>& GenSharedIterator::get() const
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return _iterCurr->get() ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedIterator::next()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  do {
    if (_iterCurr->more()) {
      _iterCurr->next() ;
    }
    if ((_iterCurr->more() == 0) && (_iterCurr == _iterNew)) {
      _iterCurr = _iterSav ;
      _iterCurr->reset() ;
    }
  } while ( (_iterCurr == _iterSav) &&
	    (_iterCurr->more()) && 
	    (_iterDel->look(_iterSav->where(), 1) != -1) ) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedIterator::prev()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  do {
    if (_iterCurr->where() != 0) {
      _iterCurr->prev() ;
    } else {
      if (_iterCurr == _iterSav) {
	_iterCurr = _iterNew ;
      }
    }
  } while ( (_iterCurr == _iterSav) &&
	    (_iterDel->look(_iterSav->where(), 1) != -1) ) ;

}


/*= EXTERNAL FUNCTION =======================================================*/
void GenSharedIterator::reset()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  _iterSav->reset() ;
  _iterNew->reset() ;
  _iterCurr = _iterNew ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int GenSharedIterator::more() const 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return (_iterNew->more()) || (_iterSav->more()) ;
}




