/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Database server
*  SCOPE :              Database server
*  FILE :               dbserver.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __rename_H
#include "rename.h"
#endif

#ifndef __dbserver_HXX
#include "dbserver.hxx"
#endif

#ifndef __dbvue_HXX
#include "dbvue.hxx"
#endif

#ifndef __volume_HXX
#include "volume.hxx"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#ifndef __list_HXX
#include "list.hxx"
#endif

#ifndef __log_HXX
#include "log.hxx"
#endif

#ifndef __page_HXX
#include "page.hxx"
#endif

/*****************************************************************************/


/*= EXTERNAL FUNCTION =======================================================*/
DBServer::DBServer(const char* n) : _name(strdup(n)), _log(NULL)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  // init tables
  for (int i = 0; i < MAX_VOL; this->_tabvol[i] = NULL, i++) ;
  for (i = 0; i < MAX_VUE; this->_tabvue[i] = NULL, i++) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
DBServer::~DBServer()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  for (int i = 0; i < MAX_VOL; i++) {
    if (this->_tabvol[i] != NULL) delete this->_tabvol[i] ; 
  }
// Vues are already deleted by VirtualDBServer
// Except in yoo_server but we don't care     
// for (i = 0; i < MAX_VUE; i++) {
//  if (this->_tabvue[i] != NULL) delete this->_tabvue[i] ;
// }
  delete _name ;
  if (_log != NULL) delete _log ;
  IterPageSet iter(this->_lock) ;
  for (; iter.more(); iter.next()) {
    delete iter.get() ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void DBServer::setVolume(ListPtr<Volume>* list_vol)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  IterListPtr<Volume> iter(*list_vol) ;
  for (; iter.more(); iter.next()) {
    _tabvol[iter.get()->volid()] = iter.get() ;
    iter.get()->open() ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DBServer::registerVue(ServerDBVue* vue)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  for (int i = 0; i < MAX_VUE; i++) {
    if (_tabvue[i] == NULL) { _tabvue[i] = vue; return i ; }
  }
  return -1 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void DBServer::keepLocks(int vue, ListVal<PID>& in, 
			 ListVal<PID>& keep, ListVal<PID>& eject)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PageHdlLock* pg ;
  IterListVal<PID> iter(in) ;
  for (; iter.more(); iter.next()) {
    pg = (DBServer::PageHdlLock*)_lock[iter.get()] ;
    if (pg == NULL) {
      pg = new DBServer::PageHdlLock(iter.get()) ;
      _lock += pg ;
      pg->upgradeLock(vue, YOO_OLD) ;
      keep += iter.get() ;
    } else if (pg->setLock(vue, YOO_OLD)) {
      keep += iter.get() ;
    } else {
      eject += iter.get() ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void DBServer::releaseLocks(int vue, ListVal<PID>& l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  IterListVal<PID> iter(l) ;
  for (; iter.more(); iter.next()) {
    this->unlock(vue, iter.get()) ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DBServer::setLock(int vue, const PID& pid, Lock l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PageHdlLock* pg = (PageHdlLock*)_lock[pid] ;
  if (pg == NULL) {
    pg = new PageHdlLock(pid) ;
    _lock += pg ;
    pg->upgradeLock(vue, l) ;
    return TRUE ;
  } else if (pg->setLock(vue, l) == FALSE) {
    for (int i = 0; i < MAX_VUE; i++) {
      if (pg->lock(i) == YOO_OLD) this->_tabvue[i]->garbage(pg->ident());
    }
    return pg->setLock(vue, l) ;
  } else {
    return TRUE ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void DBServer::unlock(int vue, const PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PageHdlLock* pg = (PageHdlLock*)_lock[pid] ;
  if (pg != NULL) {
    if (pg->unlock(vue) == 0) { _lock -= pid; delete pg ; }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DBServer::PageHdlLock::setLock(int vue, Lock l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  for (int i = 0; i < MAX_VUE; i++) {
    if (i == vue) continue ; 
    if (checklock(this->_locktable[i], l) == YOO_UNLOCK) return FALSE ;
  }
  this->_locktable[vue] = l ;
  return TRUE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int DBServer::PageHdlLock::checklock(Lock curr, Lock l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  switch (curr) {
  case YOO_EXCL:
    l = YOO_UNLOCK ;
    break ;
  case YOO_OLD :
  case YOO_SHARE :
    l = (l < YOO_EXCL) ? l : YOO_UNLOCK ;
    break ;
  case YOO_UNLOCK :
    break ;
  } 
  return l ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int DBServer::PageHdlLock::unlock(int vue)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->_locktable[vue] = YOO_UNLOCK ; 
  for (int i = 0; i < MAX_VUE; i++) {
    if (this->_locktable[i] != YOO_UNLOCK) return 1 ;
  }
  return 0 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void DBServer::cleanLog()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  // Look for partial commit during last session
  // Could append after interruption of the process (Segv, Ctrl-C, Kill...)
  char logfile[256] ;
  for (int i = 0; i < MAX_VUE; i++) {
    sprintf(logfile,"%s/.yoo_%s_%d_log", this->_log, this->_name, i) ;
    if (access(logfile, 0) == 0) {
      Log* log = new Log(logfile) ;
      this->undo(log) ;
      delete log ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void DBServer::undo(Log* log)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PID pid ;
  Page p ;
  for (int i = 0; i < log->size(); i++) {
    pid = log->read(&p, i) ;
    if (this->_tabvol[pid.volNo] != NULL) {
      this->_tabvol[pid.volNo]->write(&p, pid) ;
    }
  }
}
