/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Database vue
*  SCOPE :              Database vue
*  FILE :               dbvue.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#include <pwd.h>

#ifndef __dbvue_HXX
#include "dbvue.hxx"
#endif

#ifndef __dbserver_HXX
#include "dbserver.hxx"
#endif

#ifndef __log_HXX
#include "log.hxx"
#endif

#ifndef __volume_HXX
#include "volume.hxx"
#endif

#include "dbvue_svc.hxx"

/*****************************************************************************/


/*= INTERNAL FUNCTION =======================================================*/
void ServerDBVue::flushWrite()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (_writePages.count() == 0) return ;

  IterListVal<WritePage> iter(_writePages) ;
  PID pid ;
  for (; iter.more(); iter.next()) {
    pid = iter.get().pid ;
    Page p ;
    YOO_DBServer->volume(pid.volNo)->read(&p, pid) ;
    this->_log->write(&p, pid) ;
  }
  this->_log->sync() ;
  iter.reset() ;
  for (; iter.more(); iter.next()) {
    pid = iter.get().pid ;
    YOO_DBServer->volume(pid.volNo)->write((Page*)iter.get().buffer, pid) ;
  }
  this->_writePages.clear() ;
}


/*= INTERNAL FUNCTION =======================================================*/
int ServerDBVue::readdirect(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Volume* vol = YOO_DBServer->volume(pid.volNo) ;
  if (vol == NULL) {
    return YOO_ERR_READ ;
  }
  if (vol->read(p, pid) == -1) {    
    return YOO_ERR_READ ;
  }

  if (p->isempty()) {
    return YOO_ERR_READ ;
  } else {
    return YOO_ERR_NONE ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int ServerDBVue::sync()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->flushWrite() ;
  for (int i = 0; i < MAX_VOL; i++) {
    Volume* vol = YOO_DBServer->volume(i) ;
    if (vol != NULL) vol->sync() ;
  }
  return 0 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int ServerDBVue::secondphase()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  IterListVal<PID> iter(this->_freelist) ;
  for (; iter.more(); iter.next()) {
    Volume* vol = YOO_DBServer->volume(iter.get().volNo) ;
    if (vol != NULL) vol->freepage(iter.get()) ;
  }

  this->_alloc.clear() ;
  
  if (this->_log != NULL) this->_log->close() ;
  this->_log = NULL ;
  this->_freelist.clear() ;
  this->setRunning() ;
  return 0 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void ServerDBVue::abort(int)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  IterListVal<PID> iter(this->_alloc) ;
  for (; iter.more(); iter.next()) {
    Volume* vol = YOO_DBServer->volume(iter.get().volNo) ;
    if (vol != NULL) vol->freepage(iter.get()) ;
  }

  if (this->_log != NULL) {
    if (this->_log->isactive()) {
      YOO_DBServer->undo(this->_log) ;
    }
    this->_log->close() ;
    this->_log = NULL ;
  }

  this->_alloc.clear() ;
  this->_writePages.clear() ;
  this->_freelist.clear() ;
  this->setRunning() ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void ServerDBVue::write(const Page& page, const PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->getState() != YOO_CHECKPOINT) {
    this->setCheckpoint() ;
    this->createLog() ;
  }

  WritePage* wp = new WritePage(&page, pid) ;
  _writePages.insertRef(wp, 0) ;
  if (_writePages.count() > YOO_MAX_BUFF_PAGE) {
    this->flushWrite() ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void ServerDBVue::freepage(const PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Page p ;
  this->write(p, pid) ;
  this->_freelist += pid ;
}


/*= EXTERNAL FUNCTION =======================================================*/
PID ServerDBVue::allocation(int volNo, int num)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Volume* vol = YOO_DBServer->volume(volNo) ;
  if (vol == NULL) {
    return NULL_PID ;
  }
  PID pid = vol->alloc(num) ;
  pid.setsize(num) ;
  this->_alloc += pid ;
  return pid ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalDBVue::connect(char* name, int delta)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->setRunning() ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalDBVue::upgrade(ListVal<PID>& in, ListVal<PID>& out)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalDBVue::release(int, ListVal<PID>& keep,
			 ListVal<PID>& keep_out, 
			 ListVal<PID>& eject_out)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  keep_out.capture(keep) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalDBVue::readfirst(Page& p, const PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int err = readdirect(&p, pid) ;
  p.setError(err) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalDBVue::readnext(Page& p, const PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int err = readdirect(&p, pid) ;
//  p.setError(err) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalDBVue::lock(const PID& pid, const Lock& l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return TRUE ;
}


/*= INTERNAL FUNCTION =======================================================*/
int LocalDBVue::svcerror()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return FALSE ;
}


/*= INTERNAL FUNCTION =======================================================*/
int LocalDBVue::isConnected()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return TRUE ;
}


/*= INTERNAL FUNCTION =======================================================*/
void LocalDBVue::createLog()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->_log = new Log(YOO_DBServer->createLogName(this->_vue)) ;
}


/*= INTERNAL FUNCTION =======================================================*/
void LocalDBVue::garbage(const PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
}


int ProxyDBVue::svcerror() {
  return _vue.error() ;
}


int ProxyDBVue::isConnected() {
  return _vue.isConnected() ;
}


void ProxyDBVue::connect(char* name, int delta) {
  char svcname[256] ;
  sprintf(svcname,"%s_%s", YOODA_BASE, name) ;
  if (_vue.import(svcname) == -1) {
    sprintf(svcname,"%s_mono", YOODA_BASE) ;
    _vue.import(svcname) ;
  }
  struct passwd* pw = getpwuid(geteuid()) ;
  char userid[512] ;
  char hostname[512] ;
  _vue.host(hostname) ;
  sprintf(userid, "%s@%s", pw->pw_name, hostname) ;
  _vue.connect(userid, getpid(), YOO_CallbackName, delta) ;
}


void ProxyDBVue::upgrade(ListVal<PID>& in, ListVal<PID>& out) {
  _vue.upgrade(in, out) ;
}


void ProxyDBVue::release(int delta, ListVal<PID>& keep_in,
			 ListVal<PID>& keep_out, ListVal<PID>& eject) {
  _vue.release(delta, keep_in, keep_out, eject) ;
}


void ProxyDBVue::readfirst(Page& p, const PID& pid) {
  _vue.readfirst(p, pid) ;
}


void ProxyDBVue::readnext(Page& p, const PID& pid) {
  _vue.readnext(p, pid) ;
}


int ProxyDBVue::lock(const PID& pid, const Lock& l){
  return _vue.lock(pid, l) ;
}


int ProxyDBVue::sync() {
  return _vue.sync() ;
}


int ProxyDBVue::secondphase() {
  return _vue.secondphase() ;
}


void ProxyDBVue::abort(int delta) {
  _vue.abort(delta) ;
}


void ProxyDBVue::write(const Page& p, const PID& pid) {
  _vue.write(p, pid) ;
}


void ProxyDBVue::freepage(const PID& pid) {
  _vue.freepage(pid) ;
}


PID ProxyDBVue::allocation(int volno, int num) {
  return _vue.allocation(volno, num) ;
}
