/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Database vue
*  SCOPE :              Database vue
*  FILE :               dbvue.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __dbvue_HXX
#define __dbvue_HXX

/* IMPORTS *******************************************************************/

#ifndef __page_HXX
#include "page.hxx"
#endif

#ifndef __page_HXX
#include "page.hxx"
#endif

#ifndef __dbserver_HXX
#include "dbserver.hxx"
#endif

#ifndef _virtualmemory_HXX
#include "virtualmemory.hxx"
#endif

#ifndef __callbackserver_clnt_HXX
#include "callbackserver_clnt.hxx"
#endif

/*****************************************************************************/

class DBVue {

  private :
  
  ListVal<PID> _pidlist ;
    
  public :

  ListVal<PID>& pidlist() ;
  virtual int svcerror() { return 0 ; } 
  virtual int isConnected() { return 0 ; } 
  virtual void connect(char*, int delta) {} 
  virtual void upgrade(ListVal<PID>&, ListVal<PID>&) {} 
  virtual void release(int, ListVal<PID>&,ListVal<PID>&,ListVal<PID>&) {} 
  virtual void readfirst(Page& p, const PID& pid) {} 
  virtual void readnext(Page& p, const PID& pid) {} 
  virtual int lock(const PID& pid, const Lock& l) { return 0 ; }
  virtual int sync() { return 0 ; } 
  virtual int secondphase() { return 0 ; } 
  virtual void abort(int delta) {} 
  virtual void write(const Page& p, const PID&) {} 
  virtual void freepage(const PID& pid) {} 
  virtual PID allocation(int volno, int num) { return NULL_PID ; } 
} ;


class ServerDBVue : public DBVue {

  struct WritePage {
    PID pid ;
    char buffer[PAGE_SIZE] ;
    WritePage(Page const * p, PID id) : pid(id) {
      memcpy(this->buffer, p, PAGE_SIZE) ;
    }
  } ;

  protected :

  typedef enum { YOO_IDLE, YOO_RUNNING, YOO_CHECKPOINT } State ;
  
  int _vue ;
  State _state ;
  Log* _log ;
  ListVal<PID> _alloc ;
  ListVal<PID> _freelist ;
  ListVal<WritePage> _writePages ;
  void flushWrite() ;
  int readdirect(Page* p, PID pid) ;
  virtual void createLog() = 0 ;
  int getState() { return _state ; }
  void setRunning() { _state = YOO_RUNNING ; }
  void setIdle() { _state = YOO_IDLE ; }
  void setCheckpoint() { _state = YOO_CHECKPOINT ; }
  static void dummy_template() ;

  public :

  ServerDBVue() : _log(NULL) {} ;
  ~ServerDBVue() { this->ServerDBVue::abort(FALSE) ; } ;
  int vue() { return _vue ; }
  virtual void garbage(const PID& pid) = 0 ;	
  virtual int sync() ;
  virtual int secondphase() ;
  virtual void abort(int delta) ;
  virtual void write(const Page& p, const PID&) ;
  virtual void freepage(const PID& pid) ;
  virtual PID allocation(int volno, int num) ;
} ;


class LocalDBVue : public ServerDBVue {

  protected :

  void createLog() ;
  
  public :

  LocalDBVue() {
    this->_vue = YOO_DBServer->registerVue(this) ;
    this->setRunning() ;
  }
  void garbage(const PID& pid) ;	
  int isConnected() ;
  int svcerror() ;
  void connect(char*, int delta) ;
  void upgrade(ListVal<PID>&, ListVal<PID>&) ;
  void release(int, ListVal<PID>&, ListVal<PID>&, ListVal<PID>&) ; 
  void readfirst(Page& p, const PID& pid) ;
  void readnext(Page& p, const PID& pid) ;
  int lock(const PID& pid, const Lock& l);
} ;


#include "dbvue_clnt.hxx"

class ProxyDBVue : public DBVue {

  private :

  proxy(RemoteDBVue) _vue ;
  
  public :

  ProxyDBVue() { _vue.linkBreak(yoo_serverDown) ; }
  int svcerror() ;
  int isConnected() ;
  void connect(char*, int delta) ;
  void upgrade(ListVal<PID>&, ListVal<PID>&) ;
  void release(int, ListVal<PID>&, ListVal<PID>&, ListVal<PID>&) ; 
  void readfirst(Page& p, const PID& pid) ;
  void readnext(Page& p, const PID& pid) ;
  int lock(const PID& pid, const Lock& l);
  int sync() ;
  int secondphase() ;
  void abort(int delta) ;
  void write(const Page& p, const PID&) ;
  void freepage(const PID& pid) ;
  PID allocation(int volno, int num) ;
} ;


inline ListVal<PID>& DBVue::pidlist() {
  return _pidlist ;
}

inline SvcSocket& operator>>(SvcSocket& s, Lock& l) {
  u_int tmp ;
  s >> tmp ; l = (Lock)tmp ;
  return s ;
}

#endif
