/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Object management
*  SCOPE :              Object management
*  FILE :               obj.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __obj_HXX
#include "obj.hxx"
#endif

#ifndef __page_HXX
#include "page.hxx"
#endif

#ifndef __cluster_HXX
#include "cluster.hxx"
#endif

#ifndef __virtualdbserver_HXX
#include "virtualdbserver.hxx"
#endif

#ifndef __virtualmemory_HXX
#include "virtualmemory.hxx"
#endif

#ifndef __pagehdlmem_HXX
#include "pagehdlmem.hxx"
#endif

#ifndef __var_H
#include "var.h"
#endif

/*****************************************************************************/

/*= EXTERNAL FUNCTION =======================================================*/
void* ObjectX::operator new(size_t s)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
=*/
{
  return ::operator new(s) ;
}


/*= INTERNAL FUNCTION =======================================================*/
void* ObjectX::operator new(size_t s, Cluster* c, int clId, Page* p) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
=*/
{
  ObjectX* obj = NULL ;
  int ss = yoo_slotSize(s) ;

  // if page is not null and object can fit in a page, try allocation in page
  if ( (p != NULL) && (ss <= SLOT_IN_PG) ) {
    obj = p->allocObject(ss) ;
  } 
  // if failed, try in cluster
  if (obj == NULL) {
    obj = c->allocObject(ss) ;
  }
  if (obj != NULL) {
    obj->setClass(clId) ;
  }

  return obj ;
}


/*= INTERNAL FUNCTION =======================================================*/
void* ObjectX::operator new(size_t s, ObjectX* prox, int clId) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT)
[DIAGNOSTIC :]
=*/
{
  return ObjectX::operator new(s, prox->getCluster(), clId, prox->getPage()) ;
}


/*= INTERNAL FUNCTION =======================================================*/
void ObjectX::operator delete(void* addr)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT)
[DIAGNOSTIC :]
=*/
{
  ((ObjectX*)addr)->getPage()->freeObject(((ObjectX*)addr)->_oid) ;
}


/*= INTERNAL FUNCTION =======================================================*/
ObjectX* ObjectX::changeSize(int sizeSlot) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT)
[DIAGNOSTIC :]
=*/
{
  if (sizeSlot == this->getSlotSize()) return this ;

  ObjectX* obj = NULL ;
  int oldSize = this->getSlotSize() * SLOT_SIZE ;
  int oldpgs = yoo_pageSize(oldSize + PAGE_HEADER_SIZE) ;
  int newSize = sizeSlot * SLOT_SIZE ;
  int newpgs = yoo_pageSize(newSize + PAGE_HEADER_SIZE) ;
  
  if ((oldpgs > 1) && (newpgs > 1) && (oldpgs == newpgs)) {
    obj = this ;
    obj->setSlotSize(sizeSlot) ;
  } else {
    obj = this->getCluster()->allocObject(sizeSlot) ;
    OID newOid = obj->_oid ;
    memcpy(obj, this, newSize > oldSize ? oldSize : newSize) ;
    obj->_oid = newOid ;
    obj->setSlotSize(sizeSlot) ;
    // Case where the object really move
    if (obj->_oid != this->_oid) {
      this->getPage()->freeObject(this->_oid) ;
    }
  } 
  return obj ;
}


/*= INTERNAL FUNCTION =======================================================*/
Cluster* ObjectX::getCluster() 
/*=
[DESCRIPTION :]
  // Return cluster of the object	
[DIAGNOSTIC :]
=*/
{
  return this->getPage()->getCluster() ; 
}


/*= INTERNAL FUNCTION =======================================================*/
int ObjectX::isTemporary() 
/*=
[DESCRIPTION :]
// Return TRUE if the object is a temporary
[DIAGNOSTIC :]
=*/
{
  return yoo_getpage(this) == NULL ;
}


/*= INTERNAL FUNCTION =======================================================*/
int ObjectX::isPersistent() 
/*=
[DESCRIPTION :]
// Return TRUE if the object is a persistent
[DIAGNOSTIC :]
=*/
{
  return yoo_getpage(this) != NULL ;
}


/*= INTERNAL FUNCTION =======================================================*/
void ObjectX::initObj(OID oid, int sizeSlot) 
/*=
[DESCRIPTION :]
[DIAGNOSTIC :]
=*/
{
  this->_oid = oid ;
  this->setSlotSize(sizeSlot) ;
  this->setClass(UNTYPED) ;
}


/*= INTERNAL FUNCTION =======================================================*/
Page* ObjectX::getPage() 
/*=
[DESCRIPTION :]
[DIAGNOSTIC :]
=*/
{
#if !defined(__alpha) && defined(_vmpage_8192)
  return yoo_getpage(this)->page() ;
#else
  return (Page*)yoo_alignInf((unsigned long)this, PAGE_SIZE);
#endif
}


/*= INTERNAL FUNCTION =======================================================*/
int ObjectX::objsize() 
/*=
[DESCRIPTION :]
[DIAGNOSTIC :]
=*/
{
  return this->getSlotSize() * SLOT_SIZE ;
}
