/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             In Memory page management
*  SCOPE :              In Memory page management
*  FILE :               pagehdlmem.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __unistd_H
#define __unistd_H
#include <unistd.h>
#endif

#ifndef __pagehdlmem_HXX
#include "pagehdlmem.hxx"
#endif

#ifndef __virtualdbserver_HXX
#include "virtualdbserver.hxx"
#endif

#ifndef __virtualmemory_HXX
#include "virtualmemory.hxx"
#endif

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif

/*****************************************************************************/


/*= EXTERNAL FUNCTION =======================================================*/
PageHdlMem::PageHdlMem(Page* p, PID pid, Lock l, PageHdlMem* f, PageHdlMem* n)
: PageHdl(pid) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  _page = p ;
  _lock = l ;
  _kept = FALSE ;
  _del = FALSE ;
  _mempos = -1 ;
  _first = f == NULL ? this : f ;
  _next = n ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void PageHdlMem::insertCluster()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  YOO_VirtualMemory->set(this->_mempos) ;
  this->_page->getCluster()->insert(this) ;
  if (this->_lock <= YOO_SHARE) {
    YOO_VirtualMemory->clearAlloc(this->_mempos) ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void PageHdlMem::xlock()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Lock l = YOO_EXCL ;
  if (this->_first->getLock() < l) {
    int res = YOO_VirtualServer->lock(this->_first->pid(), l) ;
    if (res == YOO_ERR_UPG) call_throw(YOO_ERR_UPG) ;
    if (res == YOO_ERR_LOCK) {
      if (YOO_NO_WAIT == TRUE) {
	call_throw(YOO_ERR_LOCK) ;
      } else {
	int timeout = YOO_LOCK_TIMEOUT ;
	int cpt = 0 ;
	while (YOO_VirtualServer->lock(_first->pid(),l) != YOO_ERR_NONE) {
	  if (cpt++ == 0) {
	    DBG(cout << "Waiting for lock..." << endl) ;
	  }
	  sleep(LOCK_TIME_REDO) ;
	  timeout -= LOCK_TIME_REDO ;
	  if (timeout <= 0) {
	    call_throw(YOO_ERR_DEADLOCK, "DeadLock, exit...") ;
	  }
	}
      }
    }
    this->_first->_lock = l ;

    if (this->shortpage()) {
      this->remove() ;
      this->insertCluster() ;
    }
  }

  this->_lock = l ;
  YOO_VirtualMemory->set(this->_mempos) ;
  YOO_VirtualServer->touchPage(this) ;
  SYSCALLI(mprotect((caddr_t)this->_page, PAGE_SIZE, PROT_READ|PROT_WRITE)) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void PageHdlMem::eject()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  YOO_VirtualMemory->clear(this->_mempos) ;
  if (this->_kept != TRUE) {
    SYSCALLI(mprotect((caddr_t)this->_page, PAGE_SIZE, PROT_NONE)) ;
    this->_mempos = -1 ;
  } else {
    YOO_VirtualServer->eject(this->pid()) ;
    delete this ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void PageHdlMem::load()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->_lock == YOO_SHARE) {
    if (this->isfirst()) {   
      YOO_VirtualServer->readfirst(_page, this->pid()) ;
      if (_page->getError() == YOO_ERR_LOCK) {
	if (YOO_NO_WAIT == TRUE) {
	  YOO_VirtualMemory->eject(this) ;
	  call_throw(YOO_ERR_LOCK) ;
	} else {
	  int timeout = YOO_LOCK_TIMEOUT ;
	  int cpt = 0 ;
	  do {
	    if (cpt++ == 0) {
	      DBG(cout << "Waiting for lock..." << endl) ;
	    }
	    sleep(LOCK_TIME_REDO) ;
	    timeout -= LOCK_TIME_REDO ;
	    if (timeout <= 0) {
	      YOO_VirtualMemory->eject(this) ;
	      call_throw(YOO_ERR_DEADLOCK, "DeadLock, exit...") ;
	    }
	    YOO_VirtualServer->readfirst(_page,this->pid()) ;
	  } while (_page->getError() == YOO_ERR_LOCK) ;
	}
      } else if (_page->getError() != YOO_ERR_NONE) {	
	int err = _page->getError() ;
	YOO_VirtualMemory->eject(this) ;
	call_throw(err, "Invalid Address") ;
      }
      this->_page->receive() ;
      for (PageHdlMem* pd = this->_next ; pd != NULL; pd = pd->_next) {
	if (pd->inMemory() != -1) {
	  SYSCALLI(mprotect((caddr_t)pd->_page, PAGE_SIZE, PROT_READ)) ;
	} else {
	  SYSCALLI(mprotect((caddr_t)pd->_page, PAGE_SIZE, PROT_NONE)) ;
	}
      }
    } else {
      YOO_VirtualServer->readnext(_page, this->pid()) ;
      SYSCALLI(mprotect((caddr_t)this->_page, PAGE_SIZE, PROT_READ)) ;
    }
    YOO_CURR_PAGE = this->_page ;
  } else {
    YOO_VirtualMemory->set(this->_mempos) ;
    YOO_VirtualServer->touchPage(this) ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int PageHdlMem::consistency()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  char* deb = getenv("DEBUG") ;
  if ((deb == NULL) || (strcmp(getenv("DEBUG"), "TRUE"))) {
    return TRUE ;
  }

  if ( ((_del != TRUE) && (_del != FALSE)) ||
       (_lock > YOO_HEXCL) ||
       ((_mempos < -1) || (_mempos >= YOO_VM_SIZE)) ||
       ((_kept != TRUE) && (_kept != FALSE)) ) {
    yoo_error(YOO_ERR_DBG, "Inconsistent page handler") ;
    return FALSE ;
  }

  if ((this->prev() != NULL) && (this->prev()->next() != this)) {
    yoo_error(YOO_ERR_DBG, "Bad linked page") ;
    return FALSE ;
  }

  if ((this->next() != NULL) && (this->next()->prev() != this)) {
    yoo_error(YOO_ERR_DBG, "Bad linked page") ;
    return FALSE ;
  }

  if ((_del == TRUE) && (_lock < YOO_EXCL)) {
    yoo_error(YOO_ERR_DBG, "Inconsistent page handler") ;
    return FALSE ;
  }

  if ((_kept == TRUE) && (_mempos == -1)) {
    yoo_error(YOO_ERR_DBG, "Inconsistent page handler") ;
    return FALSE ;
  }

  if ((_kept == TRUE) && (_lock > YOO_SHARE)) {
    yoo_error(YOO_ERR_DBG, "Inconsistent page handler") ;
    return FALSE ;
  }

  if (this->_next != NULL) {
    return this->_next->consistency() ;
  } else {
    return TRUE ;
  }
}
