/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             In Memory page management
*  SCOPE :              In Memory page management
*  FILE :               pagehdlmem.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __pagehdlmem_HXX
#define __pagehdlmem_HXX

/* IMPORTS *******************************************************************/

#ifdef __hpux__
extern "C" {
#endif
#include <sys/mman.h>
#ifdef __hpux__
}
#endif
  

#ifndef __pagehdl_HXX
#include "pagehdl.hxx"
#endif

#ifndef __page_HXX
#include "page.hxx"
#endif

#ifndef __set_HXX
#include "set.hxx"
#endif

#ifndef __cluster_HXX
#include "cluster.hxx"
#endif

#ifndef __linkedlist_HXX
#include "linkedlist.hxx"
#endif

#ifndef __yootypes_H
#include "yootypes.h"
#endif

/*****************************************************************************/

class PageHdlMem : public LinkedElem, public PageHdl {

  private :

  public :
  
  Page* _page ;         // Referenced page
  Lock  _lock ;         // Current lock on the page
  char  _kept ;         // Flag to define a keepable page
  char  _del ;          // Page is deleted 
  int   _mempos ;       // Position in main memory
  PageHdlMem* _first ;  // First page if long object else page itself
  PageHdlMem* _next ;   // Next page if long object else NULL
  
  public :

  PageHdlMem(Page* p, PID pid, Lock l, PageHdlMem* first, PageHdlMem* next) ;
  ~PageHdlMem() ;
  void xlock() ;         // Set exclusive lock for this page
  Lock getLock() ;       // Return current lock on the page
  int freeSpace() ;      // Return free space in the page
  Page* page() ;         // Return referenced page
  int& inMemory() ;      // Return position in memory or -1
  void eject() ;         // Eject page from main memory
  void kept() ;          // The page can be kept
  int isfirst() ;        // Return TRUE if the page is the first page of an obj
  int iskept() ;         // Return TRUE if the page can be kept
  void refresh() ;       // The kept page is active now
  void load() ;          // Load page in memory
  int isdeleted() ;      // Return TRUE if the page has been deleted
  void deleted() ;       // Set page as deleted
  void insertCluster();  // Insert page in cluster allocation list
  int shortpage() ;      // Return TRUE if it is a short page
  PID& pid() ;           // Return pid of the object
  SetIdent ident() ;        // Return address of the page
  void setnext(PageHdlMem*); // Set the next pagehandler in case of long object
  int consistency() ;    // Test consistency
} ;


inline PageHdlMem::~PageHdlMem() {
  if (_page != NULL) {
    SYSCALLI(munmap((caddr_t)this->_page, PAGE_SIZE)) ;
  }
}

inline Lock PageHdlMem::getLock() {
  return _lock ;
}

inline int PageHdlMem::freeSpace() {
  return this->_page->freeSpace() ;
}

inline Page* PageHdlMem::page() {
  return this->_page ;
}

inline int& PageHdlMem::inMemory() {
  return this->_mempos ;
}

inline void PageHdlMem::kept() {
  this->remove() ;
  this->_lock = YOO_SHARE ;
  this->_kept = TRUE ;
}

inline int PageHdlMem::iskept() {
  return this->_kept ;
}

inline int PageHdlMem::isfirst() {
  return this == this->_first ;
}

inline void PageHdlMem::refresh() {
  SYSCALLI(mprotect((caddr_t)this->_page, PAGE_SIZE, PROT_READ)) ;
  this->_kept = FALSE ;
  this->insertCluster() ;
}

inline int PageHdlMem::isdeleted() {
  return this->_del ;
}

inline void PageHdlMem::deleted() {
  this->_del = TRUE ;
  this->eject() ;
}

inline int PageHdlMem::shortpage() {
  return ((_first == this) && (_next == NULL)) ;
}

inline PID& PageHdlMem::pid() {
  return this->PageHdl::ident() ;
}

inline SetIdent PageHdlMem::ident() {
  return this->_page ;
}

inline void PageHdlMem::setnext(PageHdlMem* n) {
  this->_next = n ;
}


#endif
