/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Client virtual memory Management
*  SCOPE :              Client virtual memory Management
*  FILE :               virtualmemory.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __virtualmemory_HXX
#define __virtualmemory_HXX

/* IMPORTS *******************************************************************/

#include <sys/types.h>

#ifdef __hpux__
extern "C" {
#endif
#include <sys/mman.h>
#ifdef __hpux__
}
#endif
  
#include <ucontext.h>
#include <memory.h>

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __config_H
#include "config.h"
#endif

#ifndef __hem_HXX
#include "hem.hxx"
#endif

#ifndef __set_HXX
#include "set.hxx"
#endif

#ifndef __pagehdlmem_HXX
#include "pagehdlmem.hxx"
#endif

#ifndef __tableofbits_HXX
#include "tableofbits.hxx"
#endif


/*****************************************************************************/

typedef HEM<SETROOT, SETNODE, PageHdlMem, SetIdent> PageAddrSet ;

class VirtualDBMemory {

  private :

  struct Mapping {
    caddr_t page ;
    int size ;
    Mapping(Page* p, int s) : page((caddr_t)p), size(s*PAGE_SIZE) {} ;
  } ;
      
  ListPtr<Mapping> _maps ;         // Virtual mappings
  int _fd ;                        // File descriptor for VM file
  int _size ;                      // Size of VM file
  TableOfBits _allocator ;         // In-memory allocation
  int _last ;                      // Position of the last ejected page
  char* _name ;                    // Name of swap file
  PageHdlMem** _inMemoryTable ;    // Table of in-memory pages
  Page* _currmap ;                 // Current mapping for VM file
  
  public :
  
  PageSet _inVMPagePID ;           // list VM pages indexed by PID
  PageAddrSet _inVMPageAddr ;      // list VM pages indexed by address
  int findAllocation() ;           // Find free place in real memory
  void physicalAllocation(PageHdlMem*) ; // Allocation in real memory
  static void dummy_template() ;
  
  public :

  VirtualDBMemory(int size) ;              
  ~VirtualDBMemory() ;
  Page* findPage(const PID&) ;     // Return page address if found by OID in VM
  PageHdlMem* findPageHdl(const PID&) ;// Return page hdl if found by OID in VM
  PageHdlMem* findPageHdl(Page*) ; // Return page hdl if found by address in VM
  PageHdlMem* allocation(PID,int, Lock);// Allocation in virtual memory
  int protect(Page*,int size, int mode); // Change protection for pages
  PageHdlMem** sendMemoryTable();  // Return array of in-memory pages
  void recvMemoryTable(PageHdlMem**); // Set array of in-memory pages
  void refresh(PageHdlMem*) ;      // Refresh keeped page
  void eject(PageHdlMem*) ;        // Eject page
#if defined sun && ! defined __svr4__
  static void segvhandler(int, int code, struct sigcontext*, char* addr) ;
#elif defined(__alpha__) || defined(__hpux__)
  static void segvhandler(int, int code, struct sigcontext*) ;
#elif defined sun && defined __svr4__
  static void segvhandler(int, siginfo_t* ctx,ucontext_t* uc);
#endif
  void reset() ;                   // Reset 
  void fullReset() ;               // Reset after abort
  int size() ;                     // Return size of the cache
  void set(int num) ;              // Set position in inMemoryTable
  void clear(int num) ;            // Clear position in inMemoryTable
  void clearAlloc(int num) ;       // Clear position in inMemoryTable
  void sync() ;
} ;


inline PageHdlMem* VirtualDBMemory::findPageHdl(Page* p) {
  return _inVMPageAddr[p] ;
}

inline Page* VirtualDBMemory::findPage(const PID& pid) {
  return ((PageHdlMem*)_inVMPagePID[pid])->page() ;
}

inline PageHdlMem* VirtualDBMemory::findPageHdl(const PID& pid) {
  return (PageHdlMem*)_inVMPagePID[pid] ;
}

inline int VirtualDBMemory::protect(Page* p, int size, int prot) {
  return mprotect((char*)p, size*PAGE_SIZE, prot) ;
}

inline PageHdlMem** VirtualDBMemory::sendMemoryTable() {
  return _inMemoryTable ;
}

inline void VirtualDBMemory::recvMemoryTable(PageHdlMem** table) {
  _inMemoryTable = table ;
}

inline void VirtualDBMemory::fullReset() {
  this->reset() ;
  memset((char*)_inMemoryTable, 0, _size*sizeof(PageHdlMem*)) ;
} 

inline int VirtualDBMemory::size() {
  return _size ;
}

inline void VirtualDBMemory::clear(int num) {
  _inMemoryTable[num] = NULL ;
  _allocator.clear(num, 1) ;
}

inline void VirtualDBMemory::clearAlloc(int num) {
  _allocator.clear(num, 1) ;
}

inline void VirtualDBMemory::set(int num) {
  _allocator.set(num, 1) ;
}

inline void VirtualDBMemory::sync() {
  msync((char*)YOO_CURR_PAGE, PAGE_SIZE, MS_ASYNC) ;
}

#endif
