/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Volume management
*  SCOPE :              Implements volumes for YOODA
*  FILE :               volume.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __rename_H
#include "rename.h"
#endif

#ifndef __volume_HXX
#include "volume.hxx"
#endif

#ifndef __string_H
#define __string_H
#include <string.h>
#endif

#ifndef __stat_H
#define __stat_H
#include <sys/stat.h>
#endif

#ifndef __fcntl_H
#define __fcntl_H
#include <fcntl.h>
#endif

#ifndef __volallocator_HXX
#include "volallocator.hxx"
#endif


/*= EXTERNAL FUNCTION =======================================================*/
void Volume::open()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  char lockfile[512] ;
  sprintf(lockfile, "%s.LCK", _volname) ;

  // test if the file exists
  if (access(_volname, 0) != 0) {
    // New file
    SYSCALLI(this->_fdl = ::open(lockfile, O_CREAT|O_RDWR,0666));
    SYSCALLI(this->_fd = ::open(_volname, O_CREAT|O_RDWR,0666));
    SYSCALLI(ftruncate(this->_fd, VolAllocator::SizeInPage * PAGE_SIZE)) ;
    this->_size = VolAllocator::SizeInPage ;
  } else {
    // Existing file
    SYSCALLI(this->_fdl = ::open(lockfile, O_RDWR));
    SYSCALLI(this->_fd = ::open(_volname, O_RDWR));
    struct stat buf ;
    SYSCALLI(fstat(this->_fd, &buf)) ;
    this->_size = (int) buf.st_size / PAGE_SIZE ;
    // Lock the file to avoid multiple access
  }
  SYSCALLIMESS(lockf(this->_fdl, F_TLOCK, 0), "volume already used") ;
  this->_allocator = new VolAllocator(this->_fd) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
Volume::~Volume()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  delete this->_allocator ;
  SYSCALLI(lockf(this->_fdl, F_ULOCK, 0)) ;
  SYSCALLI(close(this->_fdl)) ;
  SYSCALLI(close(this->_fd)) ;
  if (this->_volid == YOO_TMP_VOL) SYSCALLI(unlink(this->_volname)) ;
  delete _volname ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int Volume::read(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (pid.pageNo > this->_size) {
    return -1 ;
  } else {
    SYSCALLI(lseek(this->_fd, pid.pageNo * PAGE_SIZE, SEEK_SET));
    SYSCALLI(::read(this->_fd, p, PAGE_SIZE)) ;
    return 0 ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int Volume::write(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (pid.pageNo >= PAGE_IN_VOL) {
    return -1 ;
  } else {
    SYSCALLI(lseek(this->_fd, pid.pageNo * PAGE_SIZE, SEEK_SET));
    SYSCALLI(::write(this->_fd, p, PAGE_SIZE)) ;
    this->_size = this->_size > (pid.pageNo+1) ? this->_size : (pid.pageNo+1) ;
    return 0 ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
PID Volume::alloc(int number)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int page = this->_allocator->alloc(number) ;
  return page == -1 ? NULL_PID : PID(this->_volid, page) ;  
}


/*= EXTERNAL FUNCTION =======================================================*/
void Volume::freepage(PID& pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->_allocator->freepage(pid.pageNo) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int Volume::isused(PID pid, int len) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return this->_allocator->isused(pid.pageNo, len) ;
}


