/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Simple functions for yooda
*  SCOPE :              Simple functions for yooda
*  FILE :               yoofunc.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
  
#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#ifndef __var_H
#include "var.h"
#endif

#ifndef __yootypes_H
#include "yootypes.h"
#endif

#ifndef __pagehdlmem_HXX
#include "pagehdlmem.hxx"
#endif

#ifndef __virtualdbserver_HXX
#include "virtualdbserver.hxx"
#endif

#ifndef __dbserver_HXX
#include "dbserver.hxx"
#endif

#ifndef __virtualmemory_HXX
#include "virtualmemory.hxx"
#endif

#ifndef __sockrpc_HXX
#include "sockrpc.hxx"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#ifndef __list_HXX
#include "list.hxx"
#endif

#ifndef __var_H
#include "var.h"
#endif

#ifndef __yootypes_H
#include "yootypes.h"
#endif

#ifndef __define_H
#include "define.h"
#endif

#ifndef __cluster_HXX
#include "cluster.hxx"
#endif

#ifndef __volume_HXX
#include "volume.hxx"
#endif

#ifndef __volallocator_HXX
#include "volallocator.hxx"
#endif

#ifndef __smartp_HXX
#include "smartp.hxx"
#endif

#ifndef __callbackclient_clnt_HXX
#include "callbackclient_clnt.hxx"
#endif

extern void GenSharedCol_commitAll() ;
extern void GenSharedCol_abortAll() ;

/*****************************************************************************/


ostream& operator<<(ostream& o, const PID& pid)
{
  cout << "<" << pid.volNo << ", " << pid.pageNo << ">" ;
  cout << "(" << pid.size() << ")" ;
  return o ;
}


ostream& operator<<(ostream& o, const OID& oid)
{
  cout << "<" << oid.volNo << ", " << oid.pageNo << ", " << oid.objNo << ">" ;
  cout << "(" << oid.size() << ")" ;
  return o ;
}

void yoo_end()
{
  Cluster::reset() ;
  delete YOO_VirtualMemory ;
  delete YOO_VirtualServer ;
  if (YOO_DBServer != NULL) delete YOO_DBServer ;
  if ((YOO_Callback != NULL) && (YOO_Callback->isConnected())) {
    YOO_Callback->kill() ;
  }
}

void yoo_debug(char* s)
{
  printf("%s\n", s) ;
}

void yoo_debug(char* s, int val)
{
  printf("%s : %d\n", s, val) ;
}

int yoo_Terminate()
{
  yoo_end() ;
  exit(1) ;
  return 1 ;
}

void yoo_deltavue()
{
  YOO_VirtualServer->setdelta(TRUE) ;
}


void yoo_directvue()
{
  YOO_VirtualServer->setdelta(FALSE) ;
}


void yoo_serverDown(SvcSocket*)
{
  cerr << "Trouble with server" << endl ;
  yoo_exit(YOO_ERR_COM) ;
}

void yoo_callbackDown(SvcSocket*)
{
  cerr << "Trouble with callback process" << endl ;
  yoo_exit(YOO_ERR_COM) ;
}


void yoo_init(const char* server, int delta)
{
  yoo_sethook(yoo_end) ;

  if ( (server == NULL) || (strcmp(server, MONOUSER) != 0) ) {
    // Launch callback process
    char hostname[256] ;
    YOO_Callback = new proxy(CallbackClient)(AF_UNIX) ;
    YOO_Callback->linkBreak(yoo_callbackDown) ;
    YOO_Callback->host(hostname) ;
    sprintf(YOO_CallbackName,"%s_%s",tempnam("/usr/tmp","yoocb"),hostname);
    char command[256] ;  
    sprintf(command, "yoo_callback %s %d&", YOO_CallbackName, getpid()) ;
    system(command) ;
    for (int i = 0; i < 10; i++) {
      if (access(YOO_CallbackName, F_OK) != 0) {
	sleep(1) ;
      } else {
	break ;
      }
    }
    if (i == 10) yoo_callbackDown(NULL) ;

    if (YOO_Callback->import(YOO_CallbackName) == -1) {
      yoo_callbackDown(NULL) ;
    }
  } else {
    YOO_Callback = NULL ;
  }
    
  if (server == NULL) {
    server = "local" ;
  }
  
  YOO_DBServer = new DBServer(server) ;
  YOO_DBServer->setLog(YOO_SYSTEM_DIR) ;
  YOO_VirtualServer = new VirtualDBServer(YOODA_BASE) ;
  YOO_VirtualServer->setdelta(delta) ;
  YOO_VirtualMemory = new VirtualDBMemory(YOO_VM_SIZE) ;
  
  YOO_ADMIN_CLU.setOid(OID(YOO_ADMIN_VOL, YOO_FIRST_USED_PAGE, 0)) ;
  YOO_TMP_CLU.setOid(OID(YOO_ADMIN_VOL, YOO_FIRST_USED_PAGE, 1)) ;
  YOO_DEF_CLU.setOid(OID(YOO_ADMIN_VOL, YOO_FIRST_USED_PAGE, 2)) ;
  
  ListPtr<Volume> l_vol ;
  l_vol += new Volume(tempnam(YOO_VM_DIR, ".tvol"), YOO_TMP_VOL) ;
  YOO_DBServer->setVolume(&l_vol) ;
  YOO_VirtualServer->createVue(server, &l_vol) ;
  
  // Signals handlers

  struct sigaction sa ;
  
#if defined sun && defined __svr4__ 
  sa.sa_flags = SA_SIGINFO ;
  sa.sa_handler = ((void (*)())VirtualDBMemory::segvhandler) ;
#else
  sa.sa_handler = ((void (*) (...))VirtualDBMemory::segvhandler) ;
#endif
    
  sigaction(SIGSEGV, &sa, NULL) ;
  
#ifdef __hpux__
  sigaction(SIGBUS, &sa, NULL) ;
#endif
}
  

void yoo_env(const char* db, int verbose)
{
  if (db == NULL) {
    SYSCALLPMESS(YOODA_BASE = getenv("YOODA_BASE"),"YOODA_BASE undefined") ;
  } else {
    YOODA_BASE = strdup(db) ;
  }
  if (verbose) {
    cout << "--> YOODA MESG : <database," << db << ">" << endl ;
  }

  SYSCALLPMESS(YOODA_HOME = getenv("YOODA_HOME"), "YOODA_HOME undefined") ;

  if ((YOO_VM_DIR = getenv("YOODA_CACHE_DIR")) == NULL) {
    YOO_VM_DIR = "/tmp" ;
  }
  if (verbose) {
    cout << "--> YOODA MESG : <cache dir," << YOO_VM_DIR << ">" << endl ;
  }

  char* vmsize ;
  if ((vmsize = getenv("YOODA_CACHE_SIZE")) == NULL) {
    YOO_VM_SIZE = 4096 ;
  } else {
    YOO_VM_SIZE = atoi(vmsize) ;
  }

  if (verbose) {
    cout << "--> YOODA MESG : <cache size," << YOO_VM_SIZE << ">" << endl ;
  }
}


int yoo_connect(const char* db, const char* server, int delta, int verbose)
{
  // Communication tool initialisation
  SYSCALLI(SvcInit()) ;

  yoo_env(db, verbose) ;

  // Read 'repert.sys' information for the open database
  yoo_repert() ;

  // Global variable initialisation
  yoo_init(server, delta) ;
  
  // Read DB configuration file
  yoo_config() ;

  // Initialise refresh table to rebuild object when there are loaded.
  yoo_refreshInit() ;

  if (YOO_DBServer != NULL) YOO_DBServer->cleanLog() ;

  return 0 ;
}


/*= STATIC EXTERNAL FUNCTION ================================================*/
int yoo_disconnect() 
/*=
[DESCRIPTION :]
  When the session is over, we descativate signals handlers and clean workspace
  We must unlink temporary volume and callback socket. Disconnection are done 
  with connected servers.

[DIAGNOSTIC :]
[CALL :]
  External call by the user applications
=*/
{
  signal(SIGSEGV, (SIG_DFL) );

  if (yoo_abort() == -1) return -1 ;
  yoo_end() ;
  return 0 ;
}


/*= STATIC EXTERNAL FUNCTION=================================================*/
int yoo_commit() 
/*=
[DESCRIPTION :]
  Just indirection

[DIAGNOSTIC :]
  (RETURN)
    <TRUE>
      => Everything is OK. Commit is done
    <FALSE>
      => Trouble. Abort is done
[CALL :]
  External call by the user applications
=*/
{
  GenSharedCol_commitAll() ;
  yoo_checkpoint() ;
  PageHdlMem** keeplist = YOO_VirtualMemory->sendMemoryTable() ;
  if (YOO_VirtualServer->release(keeplist) == -1) { return -1 ; }
  YOO_VirtualMemory->reset() ;
  Cluster::reset() ;
  return 0 ;
}


/*= STATIC EXTERNAL FUNCTION=================================================*/
int yoo_abort() 
/*=
[DESCRIPTION :]
  Just indirection
[DIAGNOSTIC :]
[CALL :]
  External call by the user applications
=*/
{
  GenSharedCol_abortAll() ;
  PageHdlMem** keeplist = YOO_VirtualMemory->sendMemoryTable() ;
  if (YOO_VirtualServer->abort() == -1) return -1 ;
  YOO_VirtualMemory->reset() ;
  Cluster::reset() ;
  memset((char*)keeplist, 0, sizeof(PageHdlMem*)* YOO_VirtualMemory->size()) ;
  return 0 ;
}


/*= STATIC EXTERNAL FUNCTION=================================================*/
int yoo_checkpoint() 
/*=
[DESCRIPTION :]
  Just indirection
[DIAGNOSTIC :]
[CALL :]
  External call by the user applications
=*/
{
  if (YOO_VirtualServer->checkpoint() == -1) { yoo_abort(); return -1 ; }
  if (YOO_VirtualServer->secondphase() == -1) { yoo_abort(); return -1 ; }
  return 0 ;
}


/*= STATIC EXTERNAL FUNCTION=================================================*/
int yoo_upgrade() 
/*=
[DESCRIPTION :]
  Just indirection
[DIAGNOSTIC :]
[CALL :]
  External call by the user applications
=*/
{
  PageHdlMem** keeplist = YOO_VirtualMemory->sendMemoryTable() ;
  return YOO_VirtualServer->upgrade(keeplist) ;
}



/*= EXTERNAL FUNCTION =======================================================*/
PageHdlMem* yoo_getpage(void* addr)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Page* pg ;
#if !defined(__alpha) && defined(_vmpage_8192)
  pg = (Page*)yoo_alignInf((unsigned long)addr, 4096) ;    
  PageHdlMem* pd = YOO_VirtualMemory->findPageHdl(pg) ;
  if (pd != NULL) return pd ;
  addr -= 4096 ;
  pg = (Page*)yoo_alignInf((unsigned long)addr, 4096) ;    
  return YOO_VirtualMemory->findPageHdl(pg) ;
#endif
  pg = (Page*)yoo_alignInf((unsigned long)addr, PAGE_SIZE) ;    
  return YOO_VirtualMemory->findPageHdl(pg) ;
}
