#ifndef oo_objet_HXX
#define oo_objet_HXX

#ifndef oo_type_HXX
#include "oo_type.hxx"
#endif

struct BD_Date {
  unsigned annee : 8 ;
  unsigned mois  : 3 ;
  unsigned jour  : 5 ;
  unsigned heure : 16 ;
} ;


const int BD_TAILLE_CELLULE = 10 ;
typedef unsigned int BD_Cellule[BD_TAILLE_CELLULE] ;
typedef enum { BD_DateModif = 2, BD_DateCreation = 1} BD_DateMode;


struct BD_Relation 
{
  union {
    double dbl ;
    struct {
      int typ ;
      int obj ;
    } rel ;
  } ;
} ;

struct BD_Donnees : public ObjectX {
  public :
  int _taille ;
  int data[1] ;

  int taille() { return _taille ; }
  static BD_Donnees* ctor(int lg, BD_Objet* prox) ;
  void copyfrom(int* buff, int lgbuff) ;
  void copyto(int* buff, int debut, int lgbuff) ;
} ;


yoo_declare(BD_Donnees) ;


class BD_Objet {
friend class BD_BD ;
friend class BD_Type ;

  private :
 
  BD_Date          dateCreation ;
  BD_Date          dateModif ;
  BD_Cellule       cellule ;
  ListX<double>    relations ;
  Ref<BD_Objet>    pere ;
  ListX<BD_Objet>  fils ;
  Ref<BD_Type>     monType ;
  Ref<BD_Donnees>  donnees ;

  void forcerDate(BD_Date& date, char* datechar) ;
  void forcerDate(BD_Date& date) ;
  BD_Donnees* monObjetDonnees() { return donnees ; } 

  public :

  BD_Objet(int pos) ;

  // Remplissage de buffer.
  int ecrireBuffer(int longueur, int* buffer) ;

  // Ajoute l'objet dans la liste des objets du type
  void ajouterDansType(BD_Type* typ) {
    typ->objets += this ; this->monType = typ ;
  }
	
  // Ajoute un fils a l'objet.
  int ajouterFils(const Ref<BD_Objet>& objfils) ;

  // Ajoute une nouvelle relation a l'objet
  int ajouterRelation(const Ref<BD_Objet>& obj, int typeRel) ;

  // Lecture de la date de creation/modification d'un objet
  int lireDate(BD_DateMode, int*) ;

  // Retour 0 en erreur
  int ecrireCellule(int* bufcel) { 
    memcpy(&cellule[0], bufcel, sizeof(int) * BD_TAILLE_CELLULE); 
    return 1 ;
  }

  // Retour 0 en erreur  
  int forcerDateCreation(char* date) {
    this->forcerDate(this->dateCreation, date); return 1;
  }

  // Retour 0 en erreur
  int forcerDateModification(char* date) {
    this->forcerDate(this->dateModif, date); return 1;
  }

  // Retour -1 en erreur + remplissage de bufcel
  int lireCellule(int* bufcel) { 
    memcpy(bufcel, &cellule[0], sizeof(int) * BD_TAILLE_CELLULE); 
    return 1 ;
  }

  // Renvoie le nombre de mots lus. Renvoie 0 en cas d'erreur
  int lireBuffer(int debut, int longueur, int* buffer) ;

  // Renvoie le nombre de relation de l'objet 
  int nombreRelations() { return relations.count() ; } 

  // Renvoie le nombre de relation lu et remplissage du buffer de relations
  // dans les bornes demandees
  int lireRelations(int premier, int nombre, int* tabtype, int* tabrel) ;

  // Renvoie le nieme fils de l'objet
  BD_Objet* lireFils(int n) ;

  // Renvoie l'objet en relation en tenant compte du type de relation
  int lireRelation(int rang, int& typrel) ; 

  // Renvoie le pere de l'objet
  BD_Objet* lirePere() { return this->pere ; }

  // Suppression de l'objet et de sa descendance
  int supprimerDescendance() ;

  // Suppression d'un objet sans descendance
  int supprimerObjet() ;

  // Supprime l'objet de la liste de son type
  int supprimerDansType() ;
  
  // Renvoie l'identifiant du type de l'objet
  int lireType() { 
    return this->monType == NULL ? -1 : this->monType->typeIdent ; 
  }

  // Suppression de la relation de filialite
  int supprimerFilialite() ;

  int supprimerRelation(BD_Objet* obj, int typeRelation, BD_CodeSupp) ;

} ;

#endif
