#ifndef __sockrpc_HXX
#include "sockrpc.hxx"
#endif

#ifndef __connectd_svc_HXX
#define __connectd_svc_HXX

// YOODA : Yet another Object Oriented Database
//    Copyright (C) 1994  Eric Abecassis

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.

// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


class ConnectionD : public SvcServer {
  public :
  ConnectionD(int family=AF_INET) : SvcServer("ConnectionD", family) {};
  ConnectionD(char* n, int f) : SvcServer(n,f) {}


  public :
  void connect(int pid) ;
  void beginServer(char* service, char* filter, u_int port, int pid) ;
  void stopServer(char* service, char* filter, u_int port) ;
  void freeServer(char* service, char* filtre, u_int port) ;
  int  askServer(char* srv, char* filt, int m,  char*& h,  u_int& p);
  int  askLocalServer(char* s, char* f, int m,  char*& h,  u_int& p);
  void createDaemon(char* host) ;
  void notifyDaemon(char* host) ;
  void killDaemon(char* host) ;
  void infoLocal( ConnectdInfo& info) ;
  virtual void stopDaemon() ;

  protected :
  ListVal<int> proc_l ;
  ListPtr<ServDescr> serv_l ; 
  ListPtr<proxy(ConnectionD)> daemon_l ; 
  virtual void dispatchCreation(char*) ; 
  virtual void dispatchKill(char*) ; 
  void purgeServer(int pid) ;	

  public :
  void verifyServers() ;

  public :

  class ReqRegister_connect {
    public :
    int dummy ;
    ReqRegister_connect(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_connect,0);
    } 
  } req_connect ;
  static void svc_connect(SvcSocket& __svc) {
    int pid ;
    __svc >> pid ;
    ( ((ConnectionD*)(&__svc))->connect(pid) );
  } ;

  class ReqRegister_beginServer {
    public :
    int dummy ;
    ReqRegister_beginServer(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_beginServer,1);
    } 
  } req_beginServer ;
  static void svc_beginServer(SvcSocket& __svc) {
    char* service ;
    char* filter ;
    u_int port ;
    int pid ;
    __svc >> service >> filter >> port >> pid ;
    ( ((ConnectionD*)(&__svc))->beginServer(service, filter, port, pid) );
    delete service ;
    delete filter ;
  } ;

  class ReqRegister_stopServer {
    public :
    int dummy ;
    ReqRegister_stopServer(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_stopServer,2);
    } 
  } req_stopServer ;
  static void svc_stopServer(SvcSocket& __svc) {
    char* service ;
    char* filter ;
    u_int port ;
    __svc >> service >> filter >> port ;
    ( ((ConnectionD*)(&__svc))->stopServer(service, filter, port) );
    delete service ;
    delete filter ;
  } ;

  class ReqRegister_freeServer {
    public :
    int dummy ;
    ReqRegister_freeServer(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_freeServer,3);
    } 
  } req_freeServer ;
  static void svc_freeServer(SvcSocket& __svc) {
    char* service ;
    char* filtre ;
    u_int port ;
    __svc >> service >> filtre >> port ;
    ( ((ConnectionD*)(&__svc))->freeServer(service, filtre, port) );
    delete service ;
    delete filtre ;
  } ;

  class ReqRegister_askServer {
    public :
    int dummy ;
    ReqRegister_askServer(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_askServer,4);
    } 
  } req_askServer ;
  static void svc_askServer(SvcSocket& __svc) {
    char* srv ;
    char* filt ;
    int m ;
    char* h ;
    u_int p ;
    __svc >> srv >> filt >> m ;
    __svc << ( ((ConnectionD*)(&__svc))->askServer(srv, filt, m, h, p) ) << h << p << svcflush ;
    delete srv ;
    delete filt ;
  } ;

  class ReqRegister_askLocalServer {
    public :
    int dummy ;
    ReqRegister_askLocalServer(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_askLocalServer,5);
    } 
  } req_askLocalServer ;
  static void svc_askLocalServer(SvcSocket& __svc) {
    char* s ;
    char* f ;
    int m ;
    char* h ;
    u_int p ;
    __svc >> s >> f >> m ;
    __svc << ( ((ConnectionD*)(&__svc))->askLocalServer(s, f, m, h, p) ) << h << p << svcflush ;
    delete s ;
    delete f ;
  } ;

  class ReqRegister_createDaemon {
    public :
    int dummy ;
    ReqRegister_createDaemon(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_createDaemon,6);
    } 
  } req_createDaemon ;
  static void svc_createDaemon(SvcSocket& __svc) {
    char* host ;
    __svc >> host ;
    ( ((ConnectionD*)(&__svc))->createDaemon(host) );
    delete host ;
  } ;

  class ReqRegister_notifyDaemon {
    public :
    int dummy ;
    ReqRegister_notifyDaemon(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_notifyDaemon,7);
    } 
  } req_notifyDaemon ;
  static void svc_notifyDaemon(SvcSocket& __svc) {
    char* host ;
    __svc >> host ;
    ( ((ConnectionD*)(&__svc))->notifyDaemon(host) );
    delete host ;
  } ;

  class ReqRegister_killDaemon {
    public :
    int dummy ;
    ReqRegister_killDaemon(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_killDaemon,8);
    } 
  } req_killDaemon ;
  static void svc_killDaemon(SvcSocket& __svc) {
    char* host ;
    __svc >> host ;
    ( ((ConnectionD*)(&__svc))->killDaemon(host) );
    delete host ;
  } ;

  class ReqRegister_infoLocal {
    public :
    int dummy ;
    ReqRegister_infoLocal(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_infoLocal,9);
    } 
  } req_infoLocal ;
  static void svc_infoLocal(SvcSocket& __svc) {
    ConnectdInfo info ;
    ( ((ConnectionD*)(&__svc))->infoLocal(info) );
    __svc << info << svcflush ;
  } ;

  class ReqRegister_stopDaemon {
    public :
    int dummy ;
    ReqRegister_stopDaemon(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(ConnectionD::svc_stopDaemon,10);
    } 
  } req_stopDaemon ;
  static void svc_stopDaemon(SvcSocket& __svc) {
    ( ((ConnectionD*)(&__svc))->stopDaemon() );
  } ;
} ;


class MasterConnectionD : public ConnectionD {
  public :
  MasterConnectionD(int family=AF_INET) : ConnectionD("MasterConnectionD", family) {};
  MasterConnectionD(char* n, int f) : ConnectionD(n,f) {}


  public :
  void infoGlobal( ListVal<ConnectdInfo>& info_l) ;

  public :  
  void stopAllDaemon() ;
  void stopDaemon() ;
  
  protected :
  void dispatchCreation(char*) ; 
  void dispatchKill(char*) ; 

  public :

  class ReqRegister_infoGlobal {
    public :
    int dummy ;
    ReqRegister_infoGlobal(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(MasterConnectionD::svc_infoGlobal,11);
    } 
  } req_infoGlobal ;
  static void svc_infoGlobal(SvcSocket& __svc) {
    ListVal<ConnectdInfo> info_l ;
    ( ((MasterConnectionD*)(&__svc))->infoGlobal(info_l) );
    __svc << info_l << svcflush ;
  } ;
} ;


#endif
