/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#ifdef __GNUG__
#pragma interface
#endif

#ifndef _connectdinfo_HXX
#define _connectdinfo_HXX

#include "sockrpc.hxx"
#include "list.hxx"

extern u_int YOOC_PORT ;

int SvcInit() ;

class proxy(ConnectionD) ;

class ServDescr {
  char*  _service ;
  char*  _filter ;
  u_int _port ;
  int    _inused ;
  int    _pid ;
  
  public :

  ServDescr() {} ;
  ServDescr(char* s, char* f, u_int p, int pid) 
  : _service(strdup(s)), _filter(strdup(f)), _port(p), _inused(0), _pid(pid) {}
  ServDescr(const ServDescr& sd)
  : _service(strdup(sd._service)), _filter(strdup(sd._filter)), 
    _port(sd._port), _inused(sd._inused), _pid(sd._pid) {}


  ~ServDescr() { free(_service) ; free(_filter) ; }

  int pid() { return _pid ; }
  u_int port() { return _port ; }
  char* service() { return _service ; }
  char* filter() { return _filter ; }
  void setUsed() {  _inused = 1 ; }
  void setFree() {  _inused = 0 ; }
  int  isFree() { return _inused == 0 ; }

  int match(char* s, char* f, u_int p) ;
      
  int match(char* s, char* f) ;

  friend SvcSocket& operator<<(SvcSocket& s, ServDescr& sd) 
  {
    return s << sd._service << sd._filter << sd._port << sd._inused ;
  }

  friend SvcSocket& operator>>(SvcSocket& s, ServDescr& sd) 
  {
    return s >> sd._service >> sd._filter >> sd._port >> sd._inused ;
  }
} ;



class ConnectdInfo {
  public :
  ListVal<ServDescr> serv_l ;
  char* host ;

  ConnectdInfo() : host(NULL) {}
  ConnectdInfo(char* n) : host(strdup(n)) {}
  ~ConnectdInfo() { if (host != NULL) free(host) ; }
  friend SvcSocket& operator<<(SvcSocket& s, ConnectdInfo& cinfo) 
  {
    return s << cinfo.serv_l << cinfo.host ;
  }
  friend SvcSocket& operator>>(SvcSocket& s, ConnectdInfo& cinfo) 
  {
    return s >> cinfo.serv_l >> cinfo.host ;
  }
} ;


#include "connectd_clnt.hxx"

extern proxy(ConnectionD)* ConnectDaemon ;

#endif
