/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             HEM template
*  SCOPE :              Implements decoding management through HEM
*  FILE :               hem.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __hem_CC
#define __hem_CC

/* IMPORTS *******************************************************************/

#ifndef __hem_HXX
#include "hem.hxx"
#endif

/*****************************************************************************/

/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
ELEM* HEM<SROOT, SNODE, ELEM, IDENT>::operator[] (const IDENT& ident) 
/*=
[DESCRIPTION :]	
=*/
{
  register int ind = ident.getRootInd() ;
  
  // Leaf at the root level
  if (tableSwitch[ind] == HEM_LEAF) {
    return (ident==this->_u.tableLeaf[ind]->ident())?this->_u.tableLeaf[ind]:NULL;
  }
  
  // Node at the root level
  if (_u.tableNode[ind] != NULL) {
    register Node* nd = this->_u.tableNode[ind] ;
    ind = ident.getNodeInd(nd->level) ;
    while ((nd->tableSwitch[ind]==HEM_NODE) && (nd->_u.tableNode[ind]!=NULL)) {
      nd = nd->_u.tableNode[ind] ;
      ind = ident.getNodeInd(nd->level) ;
    } 
    
    if (nd->_u.tableLeaf[ind]!=NULL) {
      return (ident==nd->_u.tableLeaf[ind]->ident()) ? nd->_u.tableLeaf[ind] : NULL ;
    }
  }
  
  return NULL ;
}


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void HEM<SROOT, SNODE, ELEM, IDENT>::operator+=(ELEM* elem) 
/*=
[DESCRIPTION :]	
=*/
{
  register IDENT id = elem->ident() ;
  register int ind = id.getRootInd() ;
  
  if (_u.tableLeaf[ind] == NULL) {
    _u.tableLeaf[ind] = elem ;
    tableSwitch[ind] = HEM_LEAF ;
  } else if (tableSwitch[ind] == HEM_LEAF) {
    if (!(id == this->_u.tableLeaf[ind]->ident())) {
      _u.tableNode[ind] = collision(elem, _u.tableLeaf[ind], 0, NULL, ind) ;
      tableSwitch[ind] = HEM_NODE ;
    }
  } else {
    Node* nd = this->_u.tableNode[ind] ;
    ind = id.getNodeInd(nd->level) ;
    while ((nd->tableSwitch[ind]==HEM_NODE) && (nd->_u.tableNode[ind]!=NULL)) {
      nd = nd->_u.tableNode[ind] ;
      ind = id.getNodeInd(nd->level) ;
    }    

    if (nd->_u.tableLeaf[ind] == NULL) {
      nd->_u.tableLeaf[ind] = elem ;
      nd->tableSwitch[ind] = HEM_LEAF ;
    } else if (!(id == nd->_u.tableLeaf[ind]->ident())) {
      nd->_u.tableNode[ind]=collision(elem,nd->_u.tableLeaf[ind],nd->level,nd,ind);
      nd->tableSwitch[ind] = HEM_NODE ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void HEM<SROOT, SNODE, ELEM, IDENT>::operator-=(const IDENT& ident) 
/*=
[DESCRIPTION :]	
=*/
{
  int ind = ident.getRootInd() ;

  if ((tableSwitch[ind] == HEM_LEAF) && (ident==_u.tableLeaf[ind]->ident())) {
    _u.tableLeaf[ind] = NULL ;
    tableSwitch[ind] = HEM_NODE ;
  } else if (_u.tableNode[ind] != NULL) {
    Node* nd = this->_u.tableNode[ind] ;
    ind = ident.getNodeInd(nd->level) ;
    while ((nd->tableSwitch[ind]==HEM_NODE) && (nd->_u.tableNode[ind]!=NULL)) {
      nd = nd->_u.tableNode[ind] ;
      ind = ident.getNodeInd(nd->level) ;
    }    

    if ((nd->_u.tableLeaf[ind]!=NULL) && (ident==nd->_u.tableLeaf[ind]->ident())) {
      deleteNode(nd,ind, NULL) ;
    } 
  }
} 


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void HEM<SROOT, SNODE, ELEM, IDENT>::clear()
/*=
[DESCRIPTION :]	
=*/
{
  for (int bou = 0; bou < SROOT; bou++) {
    if ((this->_u.tableNode[bou]!=NULL)&&(this->tableSwitch[bou]==HEM_NODE)) {
      delete this->_u.tableNode[bou] ;
    }
    this->_u.tableNode[bou] = NULL ;
    this->tableSwitch[bou] = HEM_NODE ;
  }
}


/*= INTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
HEM_Node<SNODE, ELEM>*
HEM<SROOT, SNODE, ELEM, IDENT>::collision(ELEM* p1 , ELEM* p2, 
					  int level,
					  HEM_Node<SNODE, ELEM>* father,
					  int ind)
/*=
[DESCRIPTION :]
=*/
{
  register int bou = level + 1;

  int ind1 = p1->ident().getNodeInd(bou) ;
  int ind2 = p2->ident().getNodeInd(bou) ;

  while (ind1 == ind2) {
    bou++ ;
    ind1 = p1->ident().getNodeInd(bou) ;
    ind2 = p2->ident().getNodeInd(bou) ;
  }

  int bou2 = level + 1;
  Node* head = new Node(bou2);
  Node* node = head ;

  while (bou > (level+1)) {
    node->father = father ;
    node->ind = ind ;
    ind = p1->ident().getNodeInd(bou2) ;
    node->_u.tableNode[ind] = new Node(++bou2) ;
    father = node ;
    node = node->_u.tableNode[ind] ;
    bou-- ;
  }

  node->father = father ;
  node->ind = ind ;
  node->_u.tableLeaf[p1->ident().getNodeInd(bou2)] = p1 ;
  node->tableSwitch[p1->ident().getNodeInd(bou2)] = HEM_LEAF ;
  node->_u.tableLeaf[p2->ident().getNodeInd(bou2)] = p2 ;
  node->tableSwitch[p2->ident().getNodeInd(bou2)] = HEM_LEAF ;

  return(head) ; 
}                         


/*= INTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void HEM<SROOT, SNODE, ELEM, IDENT>::deleteNode(HEM_Node<SNODE, ELEM>* node,
						int ind, ELEM* leaf) 
/*=
[DESCRIPTION :]
=*/
{
  register int countLeaf = 0 ;
  register int countEnt = 0 ;
  register int indLeaf = 0 ;

  node->_u.tableLeaf[ind] = leaf ;
  node->tableSwitch[ind] = leaf == NULL ? HEM_NODE : HEM_LEAF ;
  for (int bou = 0; bou < SNODE; bou++) {
    if (node->tableSwitch[bou] == HEM_LEAF) {
      countLeaf++ ;
      indLeaf = bou ;
    }
    if (node->_u.tableNode[bou] != NULL)
      countEnt++ ;
  }
  if ( (countLeaf == 1) && (countEnt == 1) ) {
    if (node->father == NULL) {
      _u.tableLeaf[node->ind] = node->_u.tableLeaf[indLeaf] ;
      tableSwitch[node->ind] = HEM_LEAF ;
    } else {
      deleteNode(node->father, node->ind, node->_u.tableLeaf[indLeaf]) ;
    }
    delete node ;
  }
} 


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void IterHEM<SROOT, SNODE, ELEM, IDENT>::reset()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->move(-1, NULL) ;
  this->nextInNode(SROOT) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
ELEM* IterHEM<SROOT, SNODE, ELEM, IDENT>::get()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (pos.null()) {
    return NULL ;
  } else {
    return (ELEM*)(this->pos.tableNode)[pos.ind] ;
  }
}

  
/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void IterHEM<SROOT, SNODE, ELEM, IDENT>::next()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  // No more leaf
  if (this->pos.ind == -1) return ;

  // Position in a node
  while (this->pos.node != NULL) {
    HEM_Position<SNODE, ELEM> oldpos = this->pos ;
    this->nextInNode(SNODE) ;
    if (!this->pos.null()) {
      // we have found a leaf
      return ;
    }
    // go back to the father
    this->move(oldpos.node->ind, oldpos.node->father) ;
  }

  // Position in the root node
  nextInNode(SROOT) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void IterHEM<SROOT,SNODE, ELEM, IDENT>::nextInNode(int max)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  for (int i = this->pos.ind+1; i < max; i++) {
    if ( ((pos.tableNode)[i] != NULL) &&
	 ((pos.tableSwitch)[i] == HEM_LEAF) ) {
      this->move(i) ;
      // We have found a leaf
      return ;
    }
    if ( ((pos.tableNode)[i] != NULL) &&
	 ((pos.tableSwitch)[i] == HEM_NODE) ) {
      this->move(-1, pos.tableNode[i]) ;
      // Node exists only if it contains a leaf
      nextInNode(SNODE) ;
      return ;
    }
  }
  // end of iteration
  this->move(-1, NULL) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
template <int SROOT, int SNODE, class ELEM, class IDENT>
void IterHEM<SROOT,SNODE, ELEM, IDENT>::move(int i, HEM_Node<SNODE, ELEM>* node)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->pos.ind = i ;
  this->pos.node = node ;
  if (node == NULL) {
    this->pos.tableNode = hem->_u.tableNode ;
    this->pos.tableSwitch = hem->tableSwitch ;
  } else {
    this->pos.tableNode = node->_u.tableNode ;
    this->pos.tableSwitch = node->tableSwitch ;
  }
}

#endif
