/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             HEM template
*  SCOPE :              Implements decoding management through HEM
*  FILE :               hem.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __hem_HXX
#define __hem_HXX

/* IMPORTS *******************************************************************/

#ifndef __memory_H
#define __memory_H
#include <memory.h>
#endif

#ifndef __stdlib_H
#define __stdlib_H
#include <stdlib.h>
#endif

/*****************************************************************************/

typedef enum { HEM_NODE = 0, HEM_LEAF = 1 } HEM_LN ;

template <int SNODE, class ELEM> class HEM_Node {
  public : 
  
  HEM_Node(int lev) : level(lev) {
    memset(_u.tableNode, 0, sizeof(HEM_Node<SNODE, ELEM>*) * SNODE);
    memset(tableSwitch, 0, sizeof(int) * SNODE);
  }
  ~HEM_Node() {
    for (int i = 0; i < SNODE; i++) {
      if ((_u.tableNode[i] != NULL) && (tableSwitch[i] == HEM_NODE)) {
	delete _u.tableNode[i] ;
      }
    }
  }
  
  HEM_Union<ELEM, SNODE, SNODE> _u ;
  HEM_LN      tableSwitch[SNODE] ;  // Flag to distinguish node and leaf
  HEM_Node<SNODE, ELEM>*   father ;              // Father of the node
  int         ind ;                 // Position in father
  int         level ;               // Level of the node
} ;

template <class ELEM, int SROOT, int SNODE> union HEM_Union {
  HEM_Node<SNODE, ELEM>* tableNode[SROOT] ;   
  ELEM*     tableLeaf[SROOT] ;
} ; 


template <int SROOT, int SNODE, class ELEM, class IDENT> class HEM {
friend class IterHEM<SROOT, SNODE, ELEM, IDENT> ;
  
  typedef HEM_Node<SNODE, ELEM> Node ;
  
  // Local class definition for node
  
  public :

  HEM() {
    memset(_u.tableNode, 0, sizeof(HEM_Node<SNODE, ELEM>*) * SROOT);
    memset(tableSwitch, 0, sizeof(int) * SROOT);
  }
  ~HEM() { this->clear() ; }

  void      operator+=(ELEM*) ;   // Insert a new element
  void      operator-=(const IDENT&);// Retreive element referenced by IDENT
  ELEM*     operator[](const IDENT&) ; // Get the element referenced by IDENT
  void      clear() ;             // Clear the structure
  
  private : 
  
  HEM_Node<SNODE, ELEM>* collision(ELEM*, ELEM*,int,HEM_Node<SNODE,ELEM>*,int);
  void      deleteNode(HEM_Node<SNODE, ELEM>*, int, ELEM*) ;

  HEM_Union<ELEM, SROOT, SNODE> _u ;

  HEM_LN      tableSwitch[SROOT] ; // Node or Leaf
} ;



template <int SNODE, class ELEM> class HEM_Position {
  public :
  
  int ind ;
  HEM_Node<SNODE, ELEM>* node ;
  HEM_Node<SNODE, ELEM>** tableNode ;
  HEM_LN* tableSwitch ;
  
  int null() { return (ind == -1) ; }
} ;


template <int SROOT, int SNODE, class ELEM, class IDENT> class IterHEM {

  private :
  HEM_Position<SNODE, ELEM> pos ;
  HEM<SROOT, SNODE, ELEM, IDENT>* hem ;
  void move(int i) { pos.ind = i ; }
  void move(int i, HEM_Node<SNODE, ELEM>* node) ;
  void nextInNode(int max) ;
  
  public :
  void reset() ;
  IterHEM(HEM<SROOT, SNODE, ELEM, IDENT>& h) : hem(&h) { this->reset() ; }
  void next() ;
  int more() { return !pos.null() ; }
  ELEM* get() ;
} ;

#endif


