/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             Pattern matching template
*  SCOPE :              Implements reverse pattern matching
*  FILE :               pattern.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      July 1, 1994
*  COPYRIGHTS :        (C) APIC Systeme 1994
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __pattern_HXX
#include "pattern.hxx"
#endif

/*****************************************************************************/

int PATM_Etat::insererTransition(char cle, int offs)
{
  if (_transition == NULL) {
    return insererTransitionUnique(cle, offs) ;      
  } else {
    return insererTransitionMultiple(cle, offs) ;
  }
}

int PATM_Etat::insererTransitionUnique(char cle, int offs)
{
  if (_transO1 >= TRANSITION_VALIDE) {
    if (cle == _transV1) {
      return _transO1 ;
    } else {
      this->_transition = new short[PATM_MAX_CAR] ;
      for (int i = 0; i < PATM_MAX_CAR; i++) {
	_transition[i] = TRANSITION_INVALIDE ;
      }
      this->insererTransitionMultiple(_transV1, _transO1) ;
      return this->insererTransitionMultiple(cle, offs) ;
    }
  } else {
    _transV1 = cle ;
    _transO1 = cle == PATM_JOKER ? 0 : offs ;
    return cle == PATM_JOKER ? 0 : -1 ;
  }
}

int PATM_Etat::insererTransitionMultiple(char cle, int offs)
{
  int curr = _transition[cle] ;
  if (curr >= TRANSITION_VALIDE) {
    return curr ;
  } else {
    _transition[cle] = cle == PATM_JOKER ? 0 : offs ;
    return cle == PATM_JOKER ? 0 : -1 ;
  }
}

void PATM_Etat::insererTerminal(char cle, int action)
{
  if (_terminal == NULL) {
    insererTerminalUnique(cle, action) ;      
  } else {
    insererTerminalMultiple(cle, action) ;
  }
}

void PATM_Etat::insererTerminalUnique(char cle, int action)
{
  if (_termO1 > ACTION_INVALIDE) {
    if (cle == _termV1) {
      _termO1 = action ;
    } else {
      this->_terminal = new short[PATM_MAX_CAR] ;
      for (int i = 0; i < PATM_MAX_CAR; i++) {
	_terminal[i] = ACTION_INVALIDE ;
      }
      this->insererTerminalMultiple(_termV1, _termO1) ;
      this->insererTerminalMultiple(cle, action) ;
    }
  } else {
    _termV1 = cle ;
    _termO1 = action ;
  }   
  if (cle == PATM_JOKER) this->insererTransition(cle, 0) ;
}

void PATM_Etat::insererTerminalMultiple(char cle, int action)
{
  _terminal[cle] = action ;
  if (cle == PATM_JOKER) this->insererTransition(cle, 0) ;
}

int PATM_Etat::terminal(char cle)
{
  if (_terminal != NULL) {
    int actnum = _terminal[cle] ;
    return (actnum == ACTION_INVALIDE) ? _terminal[PATM_JOKER] : actnum ;
  } else {
    return (cle == _termV1) ? _termO1 :
           (_termV1 == PATM_JOKER) ? _termO1 :
                                     ACTION_INVALIDE ;
  }
}

int PATM_Etat::transition(char cle)
{
  if (_transition != NULL) {
    register int offs = _transition[cle] ;
    return (offs != TRANSITION_INVALIDE) ? offs : _transition[PATM_JOKER] ;
  } else {
    if (cle == _transV1) {
      return _transO1 ;
    } else if (_transV1 == PATM_JOKER) {
      return _transO1 ;
    } else {
      return TRANSITION_INVALIDE ;
    }
  }
}

int PATM_Contexte::deplacer(PATM_Etat* courant)
{
  int offs=0 ;
  
  if (this->_caractere > (this->_fin)) {
    // --- sortie sur un etat terminal joker
    _fcnt = courant->terminal(PATM_JOKER) ;
    return 0 ;
    
  } else if (this->_caractere == (this->_fin)) {
    // --- sortie sur le dernier caractere
    _fcnt = courant->terminal(_cle[_caractere]) ;
    if (_fcnt != ACTION_INVALIDE) {
      return 0 ;
    } else {
      if (this->_sauveEtat == -1) {
	return 0 ;
      } else {
	this->etat = this->_sauveEtat ;
	offs = 0 ;	
      }
    }
    
  } else {
    // --- progression
    if (courant->joker()) {
      this->_sauveEtat = this->etat ;
      this->_sauveCaractere = this->_caractere ;
    }
    offs = courant->transition(_cle[_caractere]) ;
    if (offs == TRANSITION_INVALIDE) {
      if (this->_sauveEtat == -1) {
	return 0 ;
      } else {
	this->etat = this->_sauveEtat ;
	this->_caractere = this->_sauveCaractere ;
	offs = 0 ;
      }
    }
    
  }
  etat += offs ;
  _caractere++ ;
  return 1 ;
}

void PATM_Etat::dump(int n, int& opt_first, int& opt_next ,char* opt_text)
{
  int l ;
  
  if ( ((opt_first < 0) || ((opt_first >= 0) && (opt_next == n))) &&
       (_transition == NULL) && (_transO1 >= TRANSITION_VALIDE) &&
       (_terminal == NULL) && (_termO1 == ACTION_INVALIDE) ) {
    // optimisation possible
    if (opt_first < 0) {
      opt_first= n ;
    }
    l= strlen(opt_text) ;
    opt_text[l]= _transV1 ;
    opt_text[l+1]= (char) 0 ;
    opt_next= (n + _transO1) ;
    // optimisation faites, dump sera fait ulterieurement
    return ;
  } 

  if (opt_first >= 0) {
    // dump optimisation
    cout << opt_first << " : " ;
    cout << opt_text << " -> " << opt_next << ", " << endl ;
    opt_first = -1 ;
    opt_text[0]= (char) 0 ;
  }
  
  cout << n << " : "  ;
  if (_transition == NULL) {
    // dump transition unique
    if (_transO1 >= TRANSITION_VALIDE) {
      cout <<  _transV1 << " -> " << (n + _transO1)  << ", " ;
    }
  } else {
    // dump transition multiple
    int j= 0 ;
    for (int i = 0; i < PATM_MAX_CAR; i++) {
      if (_transition[i] != TRANSITION_INVALIDE) {
	cout << (char) i << " -> " << (n + _transition[i]) << ", " ;
	j++ ;
	if (j>=8) {
	  j= 0 ;
	  cout << endl ;
	  cout << n << " : "  ;
	}
      }
    }
  }
  
  if (_terminal == NULL) {
    // dump terminal unique
    if (_termO1 > ACTION_INVALIDE) {
      cout <<  _termV1 << " => #" << _termO1 <<  ", " ;
    }
    
  } else {
    // dump terminal multiple
    for (int i = 0; i < PATM_MAX_CAR; i++) {
      if (_terminal[i] != ACTION_INVALIDE) {
	cout << (char) i << " => #" << _terminal[i] <<  ", " ;
      }
    }
  }
  cout << endl ;
}
