/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             I/O multiplexing
*  SCOPE :              Implements multiplexing for sockrpc
*  FILE :               pollset.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      February 1, 1994
*  COPYRIGHTS :        (C) Apic Systemes, 1994
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation "pollset.hxx"
#endif

#include <stdio.h>
#include "pollset.hxx"

#if defined(__alpha) && !defined(__GNUG__)
#include "alphadecl.h"
#endif


PollSet::PollSet()
{
  for (int i = 0; i < MAX_SOCK_FD; i++) {
    this->set[i].fd = -1 ;
    this->set[i].events = POLLIN ;
  }
  max = -1 ;
  size = 0 ;
}


PollSet::~PollSet()
{
  IterListPtr<SvcServer> iter(this->svc_l) ;
  for (; iter.more(); iter.next()) {
    if (iter.get()->isConnected()) iter.get()->close() ;
  }
}


void PollSet::setfd(int fd) 
{
  if (fd == -1) return ;
  this->set[fd].fd = fd ;
  max = max > fd ? max : fd+1 ;
  size++ ;
}


void PollSet::unsetfd(int fd) 
{
  if (fd == -1) return ;
  this->set[fd].fd = -1 ;
  this->set[fd].revents = 0 ;
  if (size == 0) {
    max = -1 ;
  } else {    
    for (int i = MAX_SOCK_FD-1; this->set[i].fd == -1; i--) ;
    max = i+1 ;
  }
  size-- ;
}


PollSet& PollSet::operator += (SvcServer* s) 
{ 
  svc_l.insert(s, 0) ; 
  this->setfd(s->listenFd()) ;
  return *this;
}

PollSet& PollSet::operator -= (SvcServer* s) 
{
  IterListPtr<SvcServer> iter(svc_l) ;
  for (; iter.more(); iter.next()) {
    if (s == iter.get()) {
      if (s->fd() != -1) this->unsetfd(s->fd()) ;
      if (s->listenFd() != -1) this->unsetfd(s->listenFd()) ;
      iter.remove() ;
      break ;
    }
  }
  return *this;
}


SvcServer* PollSet::operator[](int fd) 
{
  IterListPtr<SvcServer> iter(svc_l) ;
  for (; iter.more(); iter.next()) {
    if ( (fd == iter.get()->fd()) || (fd == iter.get()->listenFd()) ) {
      return iter.get() ;
    }
  }
  return NULL ;
}


int PollSet::exec(int t_out) 
{
  // Look for an incoming event
  if (poll(this->set, this->max, t_out) < 0) {
    perror("==> SvcSocket : unexpected error in poll") ;
    return -1 ;
  }

  int numfd = this->max ; 
  for (int i = 0; i < numfd; i++) {
    switch (this->set[i].revents) {
    default :
      perror("==> SvcSocket : unexpected error in poll") ;
      exit(1) ;
    case 0 :
      continue ;
      break ;
    case POLLHUP|POLLIN :
      replyEvent(i) ;
      if (testfd(i)) {
	this->unsetfd(i) ;
	(*this)[i]->lostConnection() ;
      }
      break ;
    case POLLHUP :
      this->unsetfd(i) ;
      (*this)[i]->lostConnection() ;    
      break ;
    case POLLIN :
      replyEvent(i) ;
      break ;
    }
  }
  return 0 ;
}


SvcEvent PollSet::findEvent(SvcServer* svc)
{
  if (svc->isConnected() == FALSE) {
    return SVC_CONNECT ;
  }

  int dummy ;
  int size = ::recv(svc->fd(), (char*)&dummy, 4, MSG_PEEK) ;

  if (size == 0) {
    return SVC_DISCONNECT ;
  } else {
    return SVC_DATA ;
  }
}
    

int PollSet::replyEvent(int fd)
{
  SvcServer* svc = (*this)[fd] ;
  
  if (svc == NULL) { return -1 ; }

  SvcEvent ev = findEvent(svc) ;

  switch (ev) {
  case SVC_CONNECT :
    if (svc->accept() < 0) {
      return -1 ;
    }
    this->setfd(svc->fd()) ;
    break ;
  case SVC_DATA :
    do {
      svc->execRpc() ;
    } while (svc->inEmpty() == FALSE) ;
    break ;
  case SVC_DISCONNECT :
    this->unsetfd(svc->fd()) ;
    svc->lostConnection() ;    
    break ;
  }

  return 0 ;
}


void PollSet::execLoop(int t_out) 
{
  while (1) {
    exec(t_out) ;
  }
}


