/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <sys/types.h>
#include <iostream.h>
#include <stdlib.h>

extern "C" {
#include <setjmp.h>
}

#include "sockrpc.hxx"
#include "list.hxx"
#include "pollset.hxx"

#define TASK_RUNNING 0
#define TASK_IDLE 1 
#define TASK_STACK_SIZE 30000
#define TASK_LIST_MAX_FD 64

class Task ;
class PendingObject ;
class ServerTask ;

class Coroutine {

  protected :

  jmp_buf env ;

  void resume() { longjmp(env, 1) ; }
  void resume(Coroutine* cor) { if (setjmp(this->env) == 0) cor->resume() ; }
} ;


class TaskScheduler : public Coroutine {
friend class Task ;
friend void taskWaitIO(int fd, int events) ;
  
  public :

  TaskScheduler() : runningTask(0), firstTask(NULL) {} ;
  void start() ;
  int  active() { return runningTask ; }
  void setRunning(Task*) ;
  void setIdle(Task*) ;

  private :

  int   runningTask ;
  Task* firstTask ;
   
  void initStack(int depth) ;
  void stackAlloc(int depth) ;
  void schedule() ;
  void insert(Task*) ;
} ;

extern TaskScheduler taskScheduler ;


class Task : public Coroutine {
friend class TaskScheduler ;
friend class PendingObject ;
  
  public :

  Task() ;

  virtual void start()  ;
  void yield(int cpt = 0) ;
  void wait(PendingObject* obj = NULL) ;
   
  private :

  TaskScheduler* myScheduler ;
  int status ;
  int delay ;
  Task *nextTask, *prevTask ;
} ;


class PendingObject {
friend class Task ;
  
  public :

  void alert() { this->waitingTask->status = TASK_RUNNING ; }
  
  private :

  Task* waitingTask ;
} ;


class Listener : public Task {
  
  private :

  int           _numTask ;
  ServerTask*   _waitingTask[TASK_LIST_MAX_FD] ;
  pollfd        _pset[TASK_LIST_MAX_FD] ;
  
  public :
  
  Listener() ;
  void scanPoll() ;
  pollfd* taskRegister(ServerTask*) ;
  void start() ;
} ;


class ServerTask : public Task {
friend void taskWaitIO(int fd, int events) ;

  private :

  SvcServer* _server ;
  pollfd*    _afd ;

  void waitFd(int fd) ;
  void replyEvent() ;
  SvcEvent findEvent() ;
  void start() ;
  
  public :

  ServerTask(Listener*, SvcServer*) ;

  void lostConnection() ;
  int open(char* name, int flag, int mode) ;
  int lseek(int fd, off_t, int) ;
  int read(int fd, char* buffer, int nbytes) ;
  int write(int fd, char* buffer, int nbytes) ;
} ;

  
