/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>

#pragma implementation "hem.hxx"
#pragma implementation "hem.cc"

#include "hem.hxx"
#include "hem.cc"


#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>

struct timeval Tbegin;
struct timeval Tend;
struct timeval tt ;

void timeBegin()
{
  gettimeofday(&Tbegin, NULL);
}

void timeEnd()
{
  gettimeofday(&Tend, NULL);
}

double timeDiff()
{
  double res = (Tend.tv_sec - Tbegin.tv_sec) ;
  res += ((float)(Tend.tv_usec)/1000000) - ((float)(Tbegin.tv_usec)/1000000) ;
  return res ;
}

struct OidInd {
  unsigned int dummy1 : 24 ;
  unsigned int lev5 : 2 ;
  unsigned int lev4 : 3 ;
  unsigned int lev3 : 3 ;
  unsigned int lev2 : 3 ;
  unsigned int lev1 : 3 ;
  unsigned int lev0 : 16 ;
  unsigned int dummy0 : 10 ;
} ;

struct OidInfo {
  unsigned int sz      : 24 ; 	// Size in page for a page.
                                // Size in slot for an object
  unsigned int volNo   :  8 ; 	// Volume number 
  unsigned int pageNo  : 22 ;	// Page number
  unsigned int objNo   : 10 ;   // object number
} ; 


class OID {
  public :

  union {
    OidInfo info ;
    int     value[2] ;
  } ;
  
  OID() {}
  OID(int val1, int val2) { this->value[0] = val1 ; this->value[1] = val2 ; }
  inline int OID::operator==(const OID& pid) const {
    return ((this->info.volNo==pid.info.volNo)&&(this->info.pageNo==pid.info.pageNo)) ;
  }
  unsigned getRootInd() { return ((OidInd*)this)->lev0 ; }
  u_int OID::getNodeInd(int lev) {
    if (lev == 1) return ((OidInd*)this)->lev1 ;
    else if (lev == 2) return ((OidInd*)this)->lev2 ;
    else if (lev == 3) return ((OidInd*)this)->lev3 ;
    else if (lev == 4) return ((OidInd*)this)->lev4 ;
    else if (lev == 5) return ((OidInd*)this)->lev5 ;
    else if (lev == 0) return ((OidInd*)this)->lev0 ;
  }    
} ;


class Page {  
  public :

  OID oid ;
  Page(OID& o) : oid(o) {} ;
  OID& ident() { return oid ; }
} ;

typedef HEM<65536, 8, Page, OID> PageIndex ; 

PageIndex pageIndex ;

int main(int argc, char** argv) {

  int number = 0 ;
  Page* page ;

  printf("Taille d'un oid : %d\n", sizeof(OID)) ;
  printf("Taille d'un oidind : %d\n", sizeof(OidInd)) ;
  
  int cpt = atoi(argv[1]) ;
  int tab[cpt][2] ;
  for (int i = 0; i < cpt; i++) {
    int val1 = rand() ;
    int val2 = rand() ;
    tab[i][0] = val1 ;
    tab[i][1] = val2 ;
    OID oid(val1, val2) ;
    if (pageIndex[oid] == NULL) {
      page = new Page(oid) ;
      pageIndex += page ;
      number++ ;
    }
  }

  cout << "Nombre d'objets inseres : " << number << endl ;

  timeBegin() ;
  for (i = 0; i < cpt; i++) {    
    page = pageIndex[OID(tab[i][0], tab[i][1])] ;
  }
  timeEnd() ;
  printf ("Temps ecoule : %f\n", timeDiff()) ;
  
}
  

  
